@@ -55,10 +55,12 @@ typedef struct
 	int freelook;
 	int hires;
 	int jump;
+	int leveltime;
 	int mouselook;
 	int neghealth;
 	int overunder;
 	int pitch;
+	int playercoords;
 	int recoil;
 	int secretmessage;
 	int smoothlight;
@@ -98,6 +100,14 @@ extern const crispy_t *critical;
 
 extern void CheckCrispySingleplayer (boolean singleplayer);
 
+enum
+{
+	REINIT_FRAMEBUFFERS = 1,
+	REINIT_RENDERER = 2,
+	REINIT_TEXTURES = 4,
+	REINIT_ASPECTRATIO = 8,
+};
+
 enum
 {
     ASPECTRATIO_OFF,
@@ -185,4 +195,12 @@ enum
     NUM_TRANSLUCENCY
 };
 
+enum
+{
+    WIDGETS_OFF,
+    WIDGETS_AUTOMAP,
+    WIDGETS_ALWAYS,
+    NUM_WIDGETS
+};
+
 #endif
