@@ -78,7 +78,9 @@
 #define CDWALLRANGE	YELLOWRANGE
 #define THINGCOLORS	GREENS
 #define THINGRANGE	GREENRANGE
-#define SECRETWALLCOLORS (crispy->extautomap ? 252 : WALLCOLORS) // [crispy] purple
+#define SECRETWALLCOLORS	252 // [crispy] purple
+#define CRISPY_HIGHLIGHT_REVEALED_SECRETS
+#define REVEALEDSECRETWALLCOLORS	112 // [crispy] green
 #define SECRETWALLRANGE WALLRANGE
 #define GRIDCOLORS	(GRAYS + GRAYSRANGE/2)
 #define GRIDRANGE	0
@@ -225,7 +227,8 @@ static int 	f_w;
 static int	f_h;
 
 static int 	lightlev; 		// used for funky strobing effect
-static pixel_t*	fb; 			// pseudo-frame buffer
+#define fb I_VideoBuffer // [crispy] simplify
+//static pixel_t*	fb; 			// pseudo-frame buffer
 static int 	amclock;
 
 static mpoint_t m_paninc; // how far the window pans each tic (map coords)
@@ -469,7 +472,7 @@ void AM_initVariables(void)
     static event_t st_notify = { ev_keyup, AM_MSGENTERED, 0, 0 };
 
     automapactive = true;
-    fb = I_VideoBuffer;
+//  fb = I_VideoBuffer; // [crispy] simplify
 
     f_oldloc.x = INT_MAX;
     amclock = 0;
@@ -579,13 +582,8 @@ void AM_LevelInit(void)
     scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
 }
 
-void AM_ReInit(boolean rescale)
+void AM_ReInit (void)
 {
-    fb = I_VideoBuffer;
-
-    if (!rescale)
-	return;
-
     f_w = SCREENWIDTH;
     f_h = SCREENHEIGHT - (ST_HEIGHT << crispy->hires);
 
@@ -1358,6 +1356,13 @@ void AM_drawWalls(void)
 		if (crispy->extautomap &&
 		    cheating && (lines[i].frontsector->special == 9))
 		    AM_drawMline(&l, SECRETWALLCOLORS);
+#if defined CRISPY_HIGHLIGHT_REVEALED_SECRETS
+		// [crispy] draw revealed secret sector boundaries in green
+		else
+		if (crispy->extautomap &&
+		    crispy->secretmessage && (lines[i].frontsector->oldspecial == 9))
+		    AM_drawMline(&l, REVEALEDSECRETWALLCOLORS);
+#endif
 		else
 		AM_drawMline(&l, WALLCOLORS+lightlev);
 	    }
@@ -1378,6 +1383,15 @@ void AM_drawWalls(void)
 		    if (false && cheating) AM_drawMline(&l, SECRETWALLCOLORS + lightlev);
 		    else AM_drawMline(&l, WALLCOLORS+lightlev);
 		}
+#if defined CRISPY_HIGHLIGHT_REVEALED_SECRETS
+		// [crispy] draw revealed secret sector boundaries in green
+		else if (crispy->extautomap && crispy->secretmessage &&
+		    (lines[i].backsector->oldspecial == 9 ||
+		    lines[i].frontsector->oldspecial == 9))
+		{
+		    AM_drawMline(&l, REVEALEDSECRETWALLCOLORS);
+		}
+#endif
 		// [crispy] draw 2S secret sector boundaries in purple
 		else if (crispy->extautomap && cheating &&
 		    (lines[i].backsector->special == 9 ||
