@@ -318,7 +318,6 @@ boolean D_Display (void)
     // [crispy] draw neither pause pic nor menu when taking a clean screenshot
     if (crispy->cleanscreenshot)
     {
-	I_FinishUpdate ();              // page flip or blit buffer
 	return false;
     }
 
@@ -391,6 +390,7 @@ void D_BindVariables(void)
     NET_BindVariables();
 
     M_BindIntVariable("mouse_sensitivity",      &mouseSensitivity);
+    M_BindIntVariable("mouse_sensitivity_x2",   &mouseSensitivity_x2); // [crispy]
     M_BindIntVariable("mouse_sensitivity_y",    &mouseSensitivity_y); // [crispy]
     M_BindIntVariable("sfx_volume",             &sfxVolume);
     M_BindIntVariable("music_volume",           &musicVolume);
@@ -434,10 +434,12 @@ void D_BindVariables(void)
     M_BindIntVariable("crispy_freelook",        &crispy->freelook);
     M_BindIntVariable("crispy_hires",           &crispy->hires);
     M_BindIntVariable("crispy_jump",            &crispy->jump);
+    M_BindIntVariable("crispy_leveltime",       &crispy->leveltime);
     M_BindIntVariable("crispy_mouselook",       &crispy->mouselook);
     M_BindIntVariable("crispy_neghealth",       &crispy->neghealth);
     M_BindIntVariable("crispy_overunder",       &crispy->overunder);
     M_BindIntVariable("crispy_pitch",           &crispy->pitch);
+    M_BindIntVariable("crispy_playercoords",    &crispy->playercoords);
     M_BindIntVariable("crispy_recoil",          &crispy->recoil);
     M_BindIntVariable("crispy_secretmessage",   &crispy->secretmessage);
     M_BindIntVariable("crispy_smoothlight",     &crispy->smoothlight);
@@ -548,7 +550,8 @@ void D_DoomLoop (void)
                " may cause demos and network games to get out of sync.\n");
     }
 
-    if (demorecording)
+    // [crispy] no need to write a demo header in demo continue mode
+    if (demorecording && gameaction != ga_playdemo)
 	G_BeginRecording ();
 
     main_loop_started = true;
@@ -1283,7 +1286,7 @@ static void LoadIwadDeh(void)
 
         // Look for chex.deh in the same directory as the IWAD file.
         dirname = M_DirName(iwadfile);
-        chex_deh = M_StringJoin(dirname, DIR_SEPARATOR_S, "chex.deh");
+        chex_deh = M_StringJoin(dirname, DIR_SEPARATOR_S, "chex.deh", NULL);
         free(dirname);
 
         // If the dehacked patch isn't found, try searching the WAD
@@ -1321,8 +1324,8 @@ static void LoadNerveWad(void)
 
     if ((i = W_GetNumForName("map01")) != -1 &&
         (j = W_GetNumForName("map09")) != -1 &&
-        !strcasecmp(lumpinfo[i]->wad_file->basename, "nerve.wad") &&
-        !strcasecmp(lumpinfo[j]->wad_file->basename, "nerve.wad"))
+        !strcasecmp(W_WadNameForLump(lumpinfo[i]), "nerve.wad") &&
+        !strcasecmp(W_WadNameForLump(lumpinfo[j]), "nerve.wad"))
     {
 	gamemission = pack_nerve;
 	DEH_AddStringReplacement ("TITLEPIC", "INTERPIC");
@@ -1335,9 +1338,9 @@ static void LoadNerveWad(void)
         (j = W_CheckNumForName("M_EPI2")) != -1 &&
         (k = W_CheckNumForName("M_EPISOD")) != -1 &&
         (gamevariant == bfgedition ||
-        (!lumpinfo[i]->wad_file->iwad &&
-         !lumpinfo[j]->wad_file->iwad &&
-         !lumpinfo[k]->wad_file->iwad)))
+        (!W_IsIWADLump(lumpinfo[i]) &&
+         !W_IsIWADLump(lumpinfo[j]) &&
+         !W_IsIWADLump(lumpinfo[k]))))
     {
         if (strrchr(iwadfile, DIR_SEPARATOR) != NULL)
         {
@@ -1378,15 +1381,15 @@ static void LoadNerveWad(void)
 // [crispy] support loading MASTERLEVELS.WAD alongside DOOM2.WAD
 static void LoadMasterlevelsWad(void)
 {
-    int i;
+    int i, j;
 
     if (gamemission != doom2)
         return;
 
-    if ((i = W_GetNumForName("map01")),
-        !strcasecmp(lumpinfo[i]->wad_file->basename, "masterlevels.wad") &&
-        (i = W_GetNumForName("map21")),
-        !strcasecmp(lumpinfo[i]->wad_file->basename, "masterlevels.wad"))
+    if ((i = W_GetNumForName("map01")) != -1 &&
+        (j = W_GetNumForName("map21")) != -1 &&
+        !strcasecmp(W_WadNameForLump(lumpinfo[i]), "masterlevels.wad") &&
+        !strcasecmp(W_WadNameForLump(lumpinfo[j]), "masterlevels.wad"))
     {
 	gamemission = pack_master;
     }
