@@ -210,6 +210,7 @@ static boolean *mousebuttons = &mousearray[1];  // allow [-1]
 
 // mouse values are used once 
 int             mousex;
+int             mousex2;
 int             mousey;         
 
 static int      dclicktime;
@@ -347,6 +348,8 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     int		forward;
     int		side;
     int		look;
+    player_t *const player = &players[consoleplayer];
+    static char playermessage[48];
 
     memset(cmd, 0, sizeof(ticcmd_t));
 
@@ -411,7 +414,6 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     // [crispy] toggle "always run"
     if (gamekeydown[key_toggleautorun])
     {
-        static char autorunmsg[24];
         static int joybspeed_old = 2;
 
         if (joybspeed >= MAX_JOY_BUTTONS)
@@ -424,15 +426,49 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
             joybspeed = 29;
         }
 
-        M_snprintf(autorunmsg, sizeof(autorunmsg), "ALWAYS RUN %s%s",
+        M_snprintf(playermessage, sizeof(playermessage), "ALWAYS RUN %s%s",
             crstr[CR_GREEN],
             (joybspeed >= MAX_JOY_BUTTONS) ? "ON" : "OFF");
-        players[consoleplayer].message = autorunmsg;
+        player->message = playermessage;
         S_StartSound(NULL, sfx_swtchn);
 
         gamekeydown[key_toggleautorun] = false;
     }
 
+    // [crispy] Toggle vertical mouse movement
+    if (gamekeydown[key_togglenovert])
+    {
+        novert = !novert;
+
+        M_snprintf(playermessage, sizeof(playermessage),
+            "vertical mouse movement %s%s",
+            crstr[CR_GREEN],
+            !novert ? "ON" : "OFF");
+        player->message = playermessage;
+        S_StartSound(NULL, sfx_swtchn);
+
+        gamekeydown[key_togglenovert] = false;
+    }
+
+    // [crispy] extra high precision IDMYPOS variant, updates for 10 seconds
+    if (player->powers[pw_mapcoords])
+    {
+        M_snprintf(playermessage, sizeof(playermessage),
+            "X=%.10f Y=%.10f A=%d",
+            (double)player->mo->x/FRACUNIT,
+            (double)player->mo->y/FRACUNIT,
+            player->mo->angle >> 24);
+        player->message = playermessage;
+
+        player->powers[pw_mapcoords]--;
+
+        // [crispy] discard instead of going static
+        if (!player->powers[pw_mapcoords])
+        {
+            player->message = "";
+        }
+    }
+
     // let movement keys cancel each other out
     if (strafe) 
     { 
@@ -666,7 +702,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
         // [crispy] released
         if (mbmlookctrl)
         {
-            if (crispy->freelook == FREELOOK_SPRING || mbmlookctrl < 6) // [crispy] short click
+            if (crispy->freelook == FREELOOK_SPRING || mbmlookctrl < SLOWTURNTICS) // [crispy] short click
             {
                 look = TOCENTER;
             }
@@ -675,7 +711,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     }
 
     if (strafe) 
-	side += mousex*2; 
+	side += mousex2*2;
     else 
 	cmd->angleturn -= mousex*0x8; 
 
@@ -686,7 +722,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
         testcontrols_mousespeed = 0;
     }
     
-    mousex = mousey = 0; 
+    mousex = mousex2 = mousey = 0;
 	 
     if (forward > MAXPLMOVE) 
 	forward = MAXPLMOVE; 
@@ -701,7 +737,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     cmd->sidemove += side;
 
     // [crispy] lookdir delta is stored in the lower 4 bits of the lookfly variable
-    if (players[consoleplayer].playerstate == PST_LIVE)
+    if (player->playerstate == PST_LIVE)
     {
         if (look < 0)
         {
@@ -818,7 +854,7 @@ void G_DoLoadLevel (void)
 
     memset (gamekeydown, 0, sizeof(gamekeydown));
     joyxmove = joyymove = joystrafemove = joylook = 0;
-    mousex = mousey = 0;
+    mousex = mousex2 = mousey = 0;
     sendpause = sendsave = paused = false;
     memset(mousearray, 0, sizeof(mousearray));
     memset(joyarray, 0, sizeof(joyarray));
@@ -991,6 +1027,10 @@ boolean G_Responder (event_t* ev)
 	mousex = ev->data2*(mouseSensitivity+5)/10; 
 	else
 	    mousex = 0; // [crispy] disable entirely
+	if (mouseSensitivity_x2)
+	mousex2 = ev->data2*(mouseSensitivity_x2+5)/10; // [crispy] separate sensitivity for strafe
+	else
+	    mousex2 = 0; // [crispy] disable entirely
 	if (mouseSensitivity_y)
 	mousey = ev->data3*(mouseSensitivity_y+5)/10; // [crispy] separate sensitivity for y-axis
 	else
@@ -1021,6 +1061,16 @@ static void G_ReadGameParms (void)
     nomonsters = M_CheckParm ("-nomonsters");
 }
  
+// [crispy] take a screenshot after rendering the next frame
+static void G_CrispyScreenShot()
+{
+	// [crispy] increase screenshot filename limit
+	V_ScreenShot("DOOM%04i.%s");
+	players[consoleplayer].message = DEH_String("screen shot");
+	crispy->cleanscreenshot = 0;
+	crispy->screenshotmsg = 2;
+}
+
 //
 // G_Ticker
 // Make ticcmd_ts for the players.
@@ -1071,18 +1121,15 @@ void G_Ticker (void)
 	    break; 
 	  case ga_screenshot: 
 	    // [crispy] redraw view without weapons and HUD
-	    if (crispy->cleanscreenshot || crispy->screenshotmsg == 1)
+	    if (gamestate == GS_LEVEL && (crispy->cleanscreenshot || crispy->screenshotmsg == 1))
 	    {
-	        extern void D_Display (void);
-
-	        crispy->screenshotmsg = 4;
-	        D_Display();
-	        I_FinishUpdate();
-	        crispy->cleanscreenshot = 0;
+		crispy->screenshotmsg = 4;
+		crispy->post_rendering_hook = G_CrispyScreenShot;
+	    }
+	    else
+	    {
+		G_CrispyScreenShot();
 	    }
-	    V_ScreenShot("DOOM%04i.%s"); // [crispy] increase screenshot filename limit
-            players[consoleplayer].message = DEH_String("screen shot");
-	    crispy->screenshotmsg = 2;
 	    gameaction = ga_nothing; 
 	    break; 
 	  case ga_nothing: 
@@ -1104,7 +1151,8 @@ void G_Ticker (void)
 
 	    if (demoplayback) 
 		G_ReadDemoTiccmd (cmd); 
-	    if (demorecording) 
+	    // [crispy] do not record tics while still playing back in demo continue mode
+	    if (demorecording && !demoplayback)
 		G_WriteDemoTiccmd (cmd);
 	    
 	    // check for turbo cheats
@@ -1198,6 +1246,7 @@ void G_Ticker (void)
     }
 
     oldgamestate = gamestate;
+    oldleveltime = leveltime;
     
     // do main actions
     switch (gamestate) 
@@ -1254,11 +1303,18 @@ void G_PlayerFinishLevel (int player)
 	 
     memset (p->powers, 0, sizeof (p->powers)); 
     memset (p->cards, 0, sizeof (p->cards)); 
+    memset (p->tryopen, 0, sizeof (p->tryopen)); // [crispy] blinking key or skull in the status bar
     p->mo->flags &= ~MF_SHADOW;		// cancel invisibility 
     p->extralight = 0;			// cancel gun flashes 
     p->fixedcolormap = 0;		// cancel ir gogles 
     p->damagecount = 0;			// no palette changes 
     p->bonuscount = 0; 
+    // [crispy] reset additional player properties
+    p->lookdir = p->oldlookdir =
+    p->centering =
+    p->jumpTics =
+    p->recoilpitch = p->oldrecoilpitch =
+    p->psp_dy = p->psp_dy_max = 0;
 } 
  
 
@@ -1887,7 +1943,9 @@ void G_DoLoadGame (void)
     // cheat codes and other single player only specifics.
     if (startloadgame == -1)
     {
+	netdemo = false;
 	netgame = false;
+	deathmatch = false;
     }
     gameaction = ga_nothing; 
 	 
@@ -1898,6 +1956,33 @@ void G_DoLoadGame (void)
         I_Error("Could not load savegame %s", savename);
     }
 
+    // [crispy] read extended savegame data
+    if (crispy->extsaveg)
+    {
+        // [crispy] first pass, read "savewadfilename"
+        P_ReadExtendedSaveGameData(0);
+    }
+    // [crispy] check if WAD file is valid to restore saved map
+    if (savewadfilename)
+    {
+        // [crispy] strings are not equal
+        if (!savemaplumpinfo ||
+            // [crispy] case-insensitive, so "doom.wad" matches "DOOM.WAD"
+            strcasecmp(savewadfilename, W_WadNameForLump(savemaplumpinfo)))
+        {
+            M_ForceLoadGame();
+            fclose(save_stream);
+            return;
+        }
+        else
+        // [crispy] strings are equal, but not identical
+        if (savewadfilename != W_WadNameForLump(savemaplumpinfo))
+        {
+            free(savewadfilename);
+        }
+    }
+    savewadfilename = NULL;
+
     savegame_error = false;
 
     if (!P_ReadSaveGameHeader())
@@ -1913,29 +1998,6 @@ void G_DoLoadGame (void)
     
     // load a base level 
     G_InitNew (gameskill, gameepisode, gamemap); 
-    // [crispy] read extended savegame data
-    if (crispy->extsaveg)
-    {
-        P_ReadExtendedSaveGameData(0);
-    }
-    // [crispy] check if WAD file is valid to restore saved map
-    if (savewadfilename)
-    {
-        // [crispy] strings are not equal
-        if (strcmp(savewadfilename, maplumpinfo->wad_file->basename))
-        {
-            M_ForceLoadGame();
-            fclose(save_stream);
-            return;
-        }
-        else
-        // [crispy] strings are equal, but not identical
-        if (savewadfilename != maplumpinfo->wad_file->basename)
-        {
-            free(savewadfilename);
-        }
-    }
-    savewadfilename = NULL;
  
     leveltime = savedleveltime;
     savedleveltime = 0;
@@ -1968,6 +2030,10 @@ void G_DoLoadGame (void)
     // do not consider it for reload
     if (players[consoleplayer].health <= 0)
 	G_ClearSavename();
+
+    // [crisy] once loaded from the command line,
+    // the next savegame will be loaded from the menu
+    startloadgame = -1;
 } 
  
 
@@ -2177,6 +2243,9 @@ G_InitNew
     if (skill > sk_nightmare)
 	skill = sk_nightmare;
 
+  // [crispy] only fix episode/map if it doesn't exist
+  if (P_GetNumForMap(episode, map, false) < 0)
+  {
     if (gameversion >= exe_ultimate)
     {
         if (episode == 0)
@@ -2213,6 +2282,7 @@ G_InitNew
       else
       map = 10;
     }
+  }
 
     M_ClearRandom ();
 
@@ -2562,7 +2632,8 @@ void G_DeferedPlayDemo(const char *name)
     gameaction = ga_playdemo; 
 
     // [crispy] fast-forward demo up to the desired map
-    if (crispy->demowarp)
+    // in demo warp mode or to the end of the demo in continue mode
+    if (crispy->demowarp || demorecording)
     {
 	nodrawers = true;
 	singletics = true;
@@ -2571,7 +2642,7 @@ void G_DeferedPlayDemo(const char *name)
 
 // Generate a string describing a demo version
 
-static char *DemoVersionDescription(int version)
+static const char *DemoVersionDescription(int version)
 {
     static char resultbuf[16];
 
@@ -2617,6 +2688,13 @@ void G_DoPlayDemo (void)
     int demoversion;
     int lumplength; // [crispy]
 
+    // [crispy] in demo continue mode free the obsolete demo buffer
+    // of size 'maxsize' previously allocated in G_RecordDemo()
+    if (demorecording)
+    {
+        Z_Free(demobuffer);
+    }
+
     lumpnum = W_GetNumForName(defdemoname);
     gameaction = ga_nothing;
     demobuffer = W_CacheLumpNum(lumpnum, PU_STATIC);
@@ -2644,14 +2722,14 @@ void G_DoPlayDemo (void)
     }
     else if (demoversion != G_VanillaVersionCode())
     {
-        char *message = "Demo is from a different game version!\n"
-                        "(read %i, should be %i)\n"
-                        "\n"
-                        "*** You may need to upgrade your version "
-                            "of Doom to v1.9. ***\n"
-                        "    See: https://www.doomworld.com/classicdoom"
-                                  "/info/patches.php\n"
-                        "    This appears to be %s.";
+        const char *message = "Demo is from a different game version!\n"
+                              "(read %i, should be %i)\n"
+                              "\n"
+                              "*** You may need to upgrade your version "
+                                  "of Doom to v1.9. ***\n"
+                              "    See: https://www.doomworld.com/classicdoom"
+                                        "/info/patches.php\n"
+                              "    This appears to be %s.";
 
         if (singledemo)
         I_Error(message, demoversion, G_VanillaVersionCode(),
@@ -2797,6 +2875,19 @@ boolean G_CheckDemoStatus (void)
 	nomonsters = false;
 	consoleplayer = 0;
         
+        // [crispy] in demo continue mode increase the demo buffer and
+        // continue recording once we are done with playback
+        if (demorecording)
+        {
+            demoend = demo_p;
+            IncreaseDemoBuffer();
+
+            nodrawers = false;
+            singletics = false;
+
+            return true;
+        }
+
         if (singledemo) 
             I_Quit (); 
         else 
