@@ -35,7 +35,7 @@
 #include "w_wad.h"
 #include "m_argv.h" // [crispy] M_ParmExists()
 #include "st_stuff.h" // [crispy] ST_HEIGHT
-#include "p_local.h" // maplumpinfo
+#include "p_setup.h" // maplumpinfo
 
 #include "s_sound.h"
 
@@ -68,7 +68,7 @@
 #define HU_INPUTWIDTH	64
 #define HU_INPUTHEIGHT	1
 
-#define HU_COORDX	(ORIGWIDTH - 8 * hu_font['A'-HU_FONTSTART]->width)
+#define HU_COORDX	(ORIGWIDTH - 7 * hu_font['A'-HU_FONTSTART]->width)
 
 
 char *chat_macros[10] =
@@ -85,7 +85,7 @@ char *chat_macros[10] =
     HUSTR_CHATMACRO9
 };
 
-char*	player_names[] =
+const char *player_names[] =
 {
     HUSTR_PLRGREEN,
     HUSTR_PLRINDIGO,
@@ -133,7 +133,7 @@ extern int		screenblocks; // [crispy]
 // The actual names can be found in DStrings.h.
 //
 
-char*	mapnames[] =	// DOOM shareware/registered/retail (Ultimate) names.
+const char *mapnames[] =	// DOOM shareware/registered/retail (Ultimate) names.
 {
 
     HUSTR_E1M1,
@@ -187,7 +187,7 @@ char*	mapnames[] =	// DOOM shareware/registered/retail (Ultimate) names.
     "NEWLEVEL"
 };
 
-char*   mapnames_chex[] =   // Chex Quest names.
+const char *mapnames_chex[] =   // Chex Quest names.
 {
 
     HUSTR_E1M1,
@@ -247,7 +247,7 @@ char*   mapnames_chex[] =   // Chex Quest names.
 // the layout in the Vanilla executable, where it is possible to
 // overflow the end of one array into the next.
 
-char *mapnames_commercial[] =
+const char *mapnames_commercial[] =
 {
     // DOOM 2 map names.
 
@@ -415,6 +415,8 @@ static void CrispyReplaceColor (char *str, const int cr, const char *col)
     free(str_replace);
 }
 
+static const char *cr_stat, *cr_stat2, *kills;
+
 void HU_Init(void)
 {
 
@@ -430,6 +432,26 @@ void HU_Init(void)
 	hu_font[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
     }
 
+    if (gameversion == exe_chex)
+    {
+	cr_stat = crstr[CR_GREEN];
+	cr_stat2 = crstr[CR_GOLD];
+	kills = "F ";
+    }
+    else
+    {
+	if (gameversion == exe_hacx)
+	{
+		cr_stat = crstr[CR_BLUE];
+	}
+	else
+	{
+		cr_stat = crstr[CR_RED];
+	}
+	cr_stat2 = crstr[CR_GREEN];
+	kills = "K ";
+    }
+
     // [crispy] initialize the crosshair types
     for (i = 0; laserpatch[i].c; i++)
     {
@@ -505,6 +527,11 @@ typedef struct
 } speciallevel_t;
 
 static const speciallevel_t speciallevels[] = {
+    // [crispy] ExM0
+    {doom, 1, 0, NULL, NULL},
+    {doom, 2, 0, NULL, NULL},
+    {doom, 3, 0, NULL, NULL},
+    {doom, 4, 0, NULL, NULL},
     // [crispy] Romero's latest E1 additions
     {doom, 1, 8, "e1m8b.wad", HUSTR_E1M8B},
     {doom, 1, 4, "e1m4b.wad", HUSTR_E1M4B},
@@ -547,7 +574,7 @@ static void HU_SetSpecialLevelName (const char *wad, const char **name)
 	    gamemap == speciallevel.map &&
 	    (!speciallevel.wad || !strcasecmp(wad, speciallevel.wad)))
 	{
-	    *name = speciallevel.name;
+	    *name = speciallevel.name ? speciallevel.name : maplumpinfo->name;
 	    break;
 	}
     }
@@ -611,7 +638,7 @@ void HU_Start(void)
 		       HU_FONTSTART);
 
     HUlib_initTextLine(&w_ltime,
-		       HU_TITLEX, HU_MSGY + 5 * 8,
+		       HU_TITLEX, HU_MSGY + 4 * 8,
 		       hu_font,
 		       HU_FONTSTART);
 
@@ -678,15 +705,15 @@ void HU_Start(void)
     }
 
     // [crispy] display names of single special levels in Automap
-    HU_SetSpecialLevelName(maplumpinfo->wad_file->basename, &s);
+    HU_SetSpecialLevelName(W_WadNameForLump(maplumpinfo), &s);
 
     // [crispy] explicitely display (episode and) map if the
     // map is from a PWAD or if the map title string has been dehacked
-    if (strcmp(s, DEH_String(s)) || (!maplumpinfo->wad_file->iwad && (!nervewadfile || gamemission != pack_nerve)))
+    if (strcmp(s, DEH_String(s)) || (!W_IsIWADLump(maplumpinfo) && (!nervewadfile || gamemission != pack_nerve)))
     {
 	char *m;
 
-	ptr = M_StringJoin(crstr[CR_GOLD], maplumpinfo->wad_file->basename, ": ", crstr[CR_GRAY], maplumpinfo->name, NULL);
+	ptr = M_StringJoin(crstr[CR_GOLD], W_WadNameForLump(maplumpinfo), ": ", crstr[CR_GRAY], maplumpinfo->name, NULL);
 	m = ptr;
 
 	while (*m)
@@ -772,8 +799,6 @@ static void HU_DrawCrosshair (void)
 void HU_Drawer(void)
 {
 
-    static char str[32], *s;
-
     if (crispy->cleanscreenshot)
     {
 	HU_Erase();
@@ -802,99 +827,34 @@ void HU_Drawer(void)
 
     if (automapactive)
     {
-	// [crispy] move map title to the bottom
-	if (crispy->automapoverlay && screenblocks >= CRISPY_HUD - 1)
-	    w_title.y = HU_TITLEY + ST_HEIGHT;
-	else
-	    w_title.y = HU_TITLEY;
-
 	HUlib_drawTextLine(&w_title, false);
     }
 
-    if (automapactive && crispy->automapstats)
+    if (crispy->automapstats == WIDGETS_ALWAYS || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
     {
-	int time = leveltime / TICRATE;
-	const char *const cr_stat = (gameversion == exe_chex) ? crstr[CR_GREEN] : crstr[CR_RED];
-	const char *const kills = (gameversion == exe_chex) ? "Flemoids: " : "Kills: ";
-
 	// [crispy] move obtrusive line out of player view
-	if (!crispy->automapoverlay || screenblocks < CRISPY_HUD - 1)
+	if (automapactive && (!crispy->automapoverlay || screenblocks < CRISPY_HUD - 1))
 	    HUlib_drawTextLine(&w_map, false);
 
-	// [crispy] count spawned monsters
-	if (extrakills)
-	    M_snprintf(str, sizeof(str), "%s%s%s%d/%d+%d", cr_stat, kills, crstr[CR_GRAY],
-	            plr->killcount, totalkills, extrakills);
-	else
-	    M_snprintf(str, sizeof(str), "%s%s%s%d/%d", cr_stat, kills, crstr[CR_GRAY],
-	            plr->killcount, totalkills);
-	HUlib_clearTextLine(&w_kills);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_kills, *(s++));
 	HUlib_drawTextLine(&w_kills, false);
-
-	M_snprintf(str, sizeof(str), "%sItems: %s%d/%d", cr_stat, crstr[CR_GRAY],
-	        plr->itemcount, totalitems);
-	HUlib_clearTextLine(&w_items);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_items, *(s++));
 	HUlib_drawTextLine(&w_items, false);
-
-	M_snprintf(str, sizeof(str), "%sSecret: %s%d/%d", cr_stat, crstr[CR_GRAY],
-	        plr->secretcount, totalsecret);
-	HUlib_clearTextLine(&w_scrts);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_scrts, *(s++));
 	HUlib_drawTextLine(&w_scrts, false);
+    }
 
-	M_snprintf(str, sizeof(str), "%s%02d:%02d:%02d", crstr[CR_GRAY],
-	        time/3600, (time%3600)/60, time%60);
-	HUlib_clearTextLine(&w_ltime);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_ltime, *(s++));
+    if (crispy->leveltime == WIDGETS_ALWAYS || (automapactive && crispy->leveltime == WIDGETS_AUTOMAP))
+    {
 	HUlib_drawTextLine(&w_ltime, false);
     }
 
-    // [crispy] show map coordinates in upper right corner
-    // if either automap stats or IDMYPOS cheat are enabled
-    if ((automapactive && crispy->automapstats) || plr->powers[pw_mapcoords])
+    if (crispy->playercoords == WIDGETS_ALWAYS || (automapactive && crispy->playercoords == WIDGETS_AUTOMAP))
     {
-	M_snprintf(str, sizeof(str), "%sX: %s%-5d", crstr[CR_GREEN], crstr[CR_GRAY],
-	        (plr->mo->x)>>FRACBITS);
-	HUlib_clearTextLine(&w_coordx);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_coordx, *(s++));
 	HUlib_drawTextLine(&w_coordx, false);
-
-	M_snprintf(str, sizeof(str), "%sY: %s%-5d", crstr[CR_GREEN], crstr[CR_GRAY],
-	        (plr->mo->y)>>FRACBITS);
-	HUlib_clearTextLine(&w_coordy);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_coordy, *(s++));
 	HUlib_drawTextLine(&w_coordy, false);
-
-	M_snprintf(str, sizeof(str), "%sA: %s%-5d", crstr[CR_GREEN], crstr[CR_GRAY],
-	        (plr->mo->angle)/ANG1);
-	HUlib_clearTextLine(&w_coorda);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_coorda, *(s++));
 	HUlib_drawTextLine(&w_coorda, false);
     }
 
     if (plr->powers[pw_showfps])
     {
-	M_snprintf(str, sizeof(str), "%s%-4d %sFPS", crstr[CR_GRAY], crispy->fps, crstr[CR_GREEN]);
-	HUlib_clearTextLine(&w_fps);
-	s = str;
-	while (*s)
-	    HUlib_addCharToTextLine(&w_fps, *(s++));
 	HUlib_drawTextLine(&w_fps, false);
     }
 
@@ -925,6 +885,10 @@ void HU_Erase(void)
     HUlib_eraseSText(&w_secret);
     HUlib_eraseIText(&w_chat);
     HUlib_eraseTextLine(&w_title);
+    HUlib_eraseTextLine(&w_kills);
+    HUlib_eraseTextLine(&w_items);
+    HUlib_eraseTextLine(&w_scrts);
+    HUlib_eraseTextLine(&w_ltime);
     HUlib_eraseTextLine(&w_coordx);
     HUlib_eraseTextLine(&w_coordy);
     HUlib_eraseTextLine(&w_coorda);
@@ -937,6 +901,7 @@ void HU_Ticker(void)
 
     int i, rc;
     char c;
+    char str[32], *s;
 
     // tick down message counter if message is up
     if (message_counter && !--message_counter)
@@ -1019,6 +984,92 @@ void HU_Ticker(void)
 	}
     }
 
+    if (automapactive)
+    {
+	// [crispy] move map title to the bottom
+	if (crispy->automapoverlay && screenblocks >= CRISPY_HUD - 1)
+	    w_title.y = HU_TITLEY + ST_HEIGHT;
+	else
+	    w_title.y = HU_TITLEY;
+    }
+
+    if (crispy->automapstats == WIDGETS_ALWAYS || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
+    {
+	// [crispy] count spawned monsters
+	if (extrakills)
+	    M_snprintf(str, sizeof(str), "%s%s%s%d/%d+%d", cr_stat, kills, crstr[CR_GRAY],
+	            plr->killcount, totalkills, extrakills);
+	else
+	    M_snprintf(str, sizeof(str), "%s%s%s%d/%d", cr_stat, kills, crstr[CR_GRAY],
+	            plr->killcount, totalkills);
+	HUlib_clearTextLine(&w_kills);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_kills, *(s++));
+
+	M_snprintf(str, sizeof(str), "%sI %s%d/%d", cr_stat, crstr[CR_GRAY],
+	        plr->itemcount, totalitems);
+	HUlib_clearTextLine(&w_items);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_items, *(s++));
+
+	M_snprintf(str, sizeof(str), "%sS %s%d/%d", cr_stat, crstr[CR_GRAY],
+	        plr->secretcount, totalsecret);
+	HUlib_clearTextLine(&w_scrts);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_scrts, *(s++));
+    }
+
+    if (crispy->leveltime == WIDGETS_ALWAYS || (automapactive && crispy->leveltime == WIDGETS_AUTOMAP))
+    {
+	const int time = leveltime / TICRATE;
+
+	if (time >= 3600)
+	    M_snprintf(str, sizeof(str), "%s%02d:%02d:%02d", crstr[CR_GRAY],
+	            time/3600, (time%3600)/60, time%60);
+	else
+	    M_snprintf(str, sizeof(str), "%s%02d:%02d", crstr[CR_GRAY],
+	            time/60, time%60);
+	HUlib_clearTextLine(&w_ltime);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_ltime, *(s++));
+    }
+
+    if (crispy->playercoords == WIDGETS_ALWAYS || (automapactive && crispy->playercoords == WIDGETS_AUTOMAP))
+    {
+	M_snprintf(str, sizeof(str), "%sX %s%-5d", cr_stat2, crstr[CR_GRAY],
+	        (plr->mo->x)>>FRACBITS);
+	HUlib_clearTextLine(&w_coordx);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_coordx, *(s++));
+
+	M_snprintf(str, sizeof(str), "%sY %s%-5d", cr_stat2, crstr[CR_GRAY],
+	        (plr->mo->y)>>FRACBITS);
+	HUlib_clearTextLine(&w_coordy);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_coordy, *(s++));
+
+	M_snprintf(str, sizeof(str), "%sA %s%-5d", cr_stat2, crstr[CR_GRAY],
+	        (plr->mo->angle)/ANG1);
+	HUlib_clearTextLine(&w_coorda);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_coorda, *(s++));
+    }
+
+    if (plr->powers[pw_showfps])
+    {
+	M_snprintf(str, sizeof(str), "%s%-4d %sFPS", crstr[CR_GRAY], crispy->fps, cr_stat2);
+	HUlib_clearTextLine(&w_fps);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_fps, *(s++));
+    }
 }
 
 #define QUEUESIZE		128
@@ -1077,7 +1128,7 @@ boolean HU_Responder(event_t *ev)
 {
 
     static char		lastmessage[HU_MAXLINELENGTH+1];
-    char*		macromessage;
+    const char		*macromessage;
     boolean		eatkey = false;
     static boolean	altdown = false;
     unsigned char 	c;
