@@ -39,6 +39,7 @@ extern multiitem_t multiitem_freelook[NUM_FREELOOKS];
 extern multiitem_t multiitem_jump[NUM_JUMPS];
 extern multiitem_t multiitem_sndchannels[4];
 extern multiitem_t multiitem_translucency[NUM_TRANSLUCENCY];
+extern multiitem_t multiitem_widgets[NUM_WIDGETS];
 
 extern void M_CrispyToggleAspectRatio(int choice);
 extern void M_CrispyToggleAutomapstats(int choice);
@@ -61,10 +62,12 @@ extern void M_CrispyToggleFreelook(int choice);
 extern void M_CrispyToggleFullsounds(int choice);
 extern void M_CrispyToggleHires(int choice);
 extern void M_CrispyToggleJumping(int choice);
+extern void M_CrispyToggleLeveltime(int choice);
 extern void M_CrispyToggleMouseLook(int choice);
 extern void M_CrispyToggleNeghealth(int choice);
 extern void M_CrispyToggleOverunder(int choice);
 extern void M_CrispyTogglePitch(int choice);
+extern void M_CrispyTogglePlayerCoords(int choice);
 extern void M_CrispyToggleRecoil(int choice);
 extern void M_CrispyToggleSecretmessage(int choice);
 extern void M_CrispyToggleSmoothLighting(int choice);
