@@ -49,8 +49,8 @@
 #include "m_argv.h"
 #include "m_controls.h"
 #include "p_saveg.h"
+#include "p_setup.h"
 #include "p_extsaveg.h" // [crispy] savewadfilename
-#include "p_local.h" // [crispy] struct maplumpinfo
 
 #include "s_sound.h"
 
@@ -73,6 +73,7 @@ extern boolean		chat_on;		// in heads-up code
 // defaulted values
 //
 int			mouseSensitivity = 5;
+int			mouseSensitivity_x2 = 5; // [crispy] mouse sensitivity menu
 int			mouseSensitivity_y = 5; // [crispy] mouse sensitivity menu
 
 // Show messages has default, 0 = off, 1 = on
@@ -181,7 +182,7 @@ short		whichSkull;		// which skull to draw
 
 // graphic name of skulls
 // warning: initializer-string for array of chars is too long
-char    *skullName[2] = {"M_SKULL1","M_SKULL2"};
+const char *skullName[2] = {"M_SKULL1","M_SKULL2"};
 
 // current menudef
 menu_t*	currentMenu;                          
@@ -203,6 +204,7 @@ static void M_QuitDOOM(int choice);
 
 static void M_ChangeMessages(int choice);
 static void M_ChangeSensitivity(int choice);
+static void M_ChangeSensitivity_x2(int choice); // [crispy] mouse sensitivity menu
 static void M_ChangeSensitivity_y(int choice); // [crispy] mouse sensitivity menu
 static void M_MouseInvert(int choice); // [crispy] mouse sensitivity menu
 static void M_SfxVol(int choice);
@@ -233,8 +235,8 @@ static void M_DrawSave(void);
 static void M_DrawSaveLoadBorder(int x,int y);
 static void M_SetupNextMenu(menu_t *menudef);
 static void M_DrawThermo(int x,int y,int thermWidth,int thermDot);
-static void M_WriteText(int x, int y, char *string);
-static int  M_StringWidth(char *string);
+static void M_WriteText(int x, int y, const char *string);
+static int  M_StringWidth(const char *string);
 static int  M_StringHeight(const char *string);
 static void M_StartMessage(const char *string, void *routine, boolean input);
 static void M_ClearMenus (void);
@@ -419,8 +421,10 @@ enum
 {
     mouse_horiz,
     mouse_empty1,
-    mouse_vert,
+    mouse_horiz2,
     mouse_empty2,
+    mouse_vert,
+    mouse_empty3,
     mouse_invert,
     mouse_end
 } mouse_e;
@@ -429,6 +433,8 @@ static menuitem_t MouseMenu[]=
 {
     {2,"",	M_ChangeSensitivity,'h'},
     {-1,"",0,'\0'},
+    {2,"",	M_ChangeSensitivity_x2,'s'},
+    {-1,"",0,'\0'},
     {2,"",	M_ChangeSensitivity_y,'v'},
     {-1,"",0,'\0'},
     {1,"",	M_MouseInvert,'i'},
@@ -450,7 +456,9 @@ enum
     crispness_sep_rendering,
     crispness_hires,
     crispness_uncapped,
+    crispness_vsync,
     crispness_smoothscaling,
+    crispness_aspectratio,
     crispness_sep_rendering_,
 
     crispness_sep_visual,
@@ -472,7 +480,9 @@ static menuitem_t Crispness1Menu[]=
     {-1,"",0,'\0'},
     {1,"",	M_CrispyToggleHires,'h'},
     {1,"",	M_CrispyToggleUncapped,'u'},
+    {1,"",	M_CrispyToggleVsync,'v'},
     {1,"",	M_CrispyToggleSmoothScaling,'s'},
+    {1,"",	M_CrispyToggleAspectRatio,'f'},
     {-1,"",0,'\0'},
     {-1,"",0,'\0'},
     {1,"",	M_CrispyToggleColoredhud,'c'},
@@ -492,7 +502,7 @@ static menu_t  Crispness1Def =
     &OptionsDef,
     Crispness1Menu,
     M_DrawCrispness1,
-    48,36,
+    48,28,
     1
 };
 
@@ -508,6 +518,8 @@ enum
     crispness_sep_navigational,
     crispness_extautomap,
     crispness_automapstats,
+    crispness_leveltime,
+    crispness_playercoords,
     crispness_secretmessage,
     crispness_sep_navigational_,
 
@@ -527,6 +539,8 @@ static menuitem_t Crispness2Menu[]=
     {-1,"",0,'\0'},
     {1,"",	M_CrispyToggleExtAutomap,'e'},
     {1,"",	M_CrispyToggleAutomapstats,'s'},
+    {1,"",	M_CrispyToggleLeveltime,'l'},
+    {1,"",	M_CrispyTogglePlayerCoords,'p'},
     {1,"",	M_CrispyToggleSecretmessage,'s'},
     {-1,"",0,'\0'},
     {1,"",	M_CrispnessNext,'n'},
@@ -539,7 +553,7 @@ static menu_t  Crispness2Def =
     &OptionsDef,
     Crispness2Menu,
     M_DrawCrispness2,
-    48,36,
+    48,28,
     1
 };
 
@@ -592,7 +606,7 @@ static menu_t  Crispness3Def =
     &OptionsDef,
     Crispness3Menu,
     M_DrawCrispness3,
-    48,36,
+    48,28,
     1
 };
 
@@ -640,7 +654,7 @@ static menu_t  Crispness4Def =
     &OptionsDef,
     Crispness4Menu,
     M_DrawCrispness4,
-    48,36,
+    48,28,
     1
 };
 
@@ -896,12 +910,6 @@ void M_LoadSelect(int choice)
 //
 void M_LoadGame (int choice)
 {
-    // [crispy] forbid New Game and (Quick) Load while recording a demo
-    if (demorecording)
-    {
-	return;
-    }
-
     // [crispy] allow loading game while multiplayer demo playback
     if (netgame && !demoplayback)
     {
@@ -955,8 +963,19 @@ void M_DoSave(int slot)
 //
 static void SetDefaultSaveName(int slot)
 {
-    M_snprintf(savegamestrings[itemOn], SAVESTRINGSIZE - 1,
-               "JOYSTICK SLOT %i", itemOn + 1);
+    // map from IWAD or PWAD?
+    if (W_IsIWADLump(maplumpinfo))
+    {
+        M_snprintf(savegamestrings[itemOn], SAVESTRINGSIZE,
+                   "%s", maplumpinfo->name);
+    }
+    else
+    {
+        M_snprintf(savegamestrings[itemOn], SAVESTRINGSIZE,
+                   "%s: %s", W_WadNameForLump(maplumpinfo),
+                   maplumpinfo->name);
+    }
+    M_ForceUppercase(savegamestrings[itemOn]);
     joypadSave = false;
 }
 
@@ -984,7 +1003,7 @@ void M_SaveSelect(int choice)
     {
         savegamestrings[choice][0] = 0;
 
-        if (joypadSave)
+        if (joypadSave || true) // [crispy] always prefill empty savegame slot names
         {
             SetDefaultSaveName(choice);
         }
@@ -1015,7 +1034,7 @@ void M_SaveGame (int choice)
 //
 //      M_QuickSave
 //
-char    tempstring[80];
+static char tempstring[90];
 
 void M_QuickSaveResponse(int key)
 {
@@ -1052,9 +1071,10 @@ void M_QuickSave(void)
                                   savegamestrings[quickSaveSlot],
                                   crstr[CR_NONE],
                                   NULL);
-    DEH_snprintf(tempstring, 80, QSPROMPT, savegamestring);
+    DEH_snprintf(tempstring, sizeof(tempstring),
+                 QSPROMPT, savegamestring);
     free(savegamestring);
-    M_StartMessage(tempstring,M_QuickSaveResponse,true);
+    M_StartMessage(tempstring, M_QuickSaveResponse, true);
 }
 
 
@@ -1097,9 +1117,10 @@ void M_QuickLoad(void)
                                   savegamestrings[quickSaveSlot],
                                   crstr[CR_NONE],
                                   NULL);
-    DEH_snprintf(tempstring, 80, QLPROMPT, savegamestring);
+    DEH_snprintf(tempstring, sizeof(tempstring),
+                 QLPROMPT, savegamestring);
     free(savegamestring);
-    M_StartMessage(tempstring,M_QuickLoadResponse,true);
+    M_StartMessage(tempstring, M_QuickLoadResponse, true);
 }
 
 
@@ -1218,7 +1239,7 @@ void M_DrawNewGame(void)
 
 void M_NewGame(int choice)
 {
-    // [crispy] forbid New Game and (Quick) Load while recording a demo
+    // [crispy] forbid New Game while recording a demo
     if (demorecording)
     {
 	return;
@@ -1299,8 +1320,8 @@ static void M_Expansion(int choice)
 //
 // [crispy] no patches are drawn in the Options menu anymore
 /*
-static char *detailNames[2] = {"M_GDHIGH","M_GDLOW"};
-static char *msgNames[2] = {"M_MSGOFF","M_MSGON"};
+static const char *detailNames[2] = {"M_GDHIGH","M_GDLOW"};
+static const char *msgNames[2] = {"M_MSGOFF","M_MSGON"};
 */
 
 void M_DrawOptions(void)
@@ -1338,18 +1359,24 @@ static void M_DrawMouse(void)
 {
     char mouse_menu_text[48];
 
-    V_DrawPatchShadow2 (60, 38, W_CacheLumpName(DEH_String("M_MSENS"), PU_CACHE));
+    V_DrawPatchShadow2 (60, LoadDef_y, W_CacheLumpName(DEH_String("M_MSENS"), PU_CACHE));
 
     M_WriteText(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_horiz + 6,
-                "HORIZONTAL");
+                "HORIZONTAL: TURN");
 
     M_DrawThermo(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_empty1,
 		 21, mouseSensitivity);
 
+    M_WriteText(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_horiz2 + 6,
+                "HORIZONTAL: STRAFE");
+
+    M_DrawThermo(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_empty2,
+		 21, mouseSensitivity_x2);
+
     M_WriteText(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_vert + 6,
                 "VERTICAL");
 
-    M_DrawThermo(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_empty2,
+    M_DrawThermo(MouseDef.x, MouseDef.y + LINEHEIGHT * mouse_empty3,
 		 21, mouseSensitivity_y);
 
     M_snprintf(mouse_menu_text, sizeof(mouse_menu_text),
@@ -1389,7 +1416,7 @@ static void M_DrawCrispnessHeader(char *item)
 {
     M_snprintf(crispy_menu_text, sizeof(crispy_menu_text),
                "%s%s", crstr[CR_GOLD], item);
-    M_WriteText(ORIGWIDTH/2 - M_StringWidth(item) / 2, 20, crispy_menu_text);
+    M_WriteText(ORIGWIDTH/2 - M_StringWidth(item) / 2, 12, crispy_menu_text);
 }
 
 static void M_DrawCrispnessSeparator(int y, char *item)
@@ -1432,8 +1459,10 @@ static void M_DrawCrispness1(void)
 
     M_DrawCrispnessSeparator(crispness_sep_rendering, "Rendering");
     M_DrawCrispnessItem(crispness_hires, "High Resolution Rendering", crispy->hires, true);
-    M_DrawCrispnessMultiItem(crispness_uncapped, "Uncapped Framerate", multiitem_uncapped, crispy->uncapped, !force_software_renderer);
+    M_DrawCrispnessItem(crispness_uncapped, "Uncapped Framerate", crispy->uncapped, true);
+    M_DrawCrispnessItem(crispness_vsync, "Enable VSync", crispy->vsync, !force_software_renderer);
     M_DrawCrispnessItem(crispness_smoothscaling, "Smooth Pixel Scaling", crispy->smoothscaling, true);
+    M_DrawCrispnessMultiItem(crispness_aspectratio, "Force Aspect Ratio", multiitem_aspectratio, aspect_ratio_correct, true);
 
     M_DrawCrispnessSeparator(crispness_sep_visual, "Visual");
     M_DrawCrispnessMultiItem(crispness_coloredhud, "Colorize HUD Elements", multiitem_coloredhud, crispy->coloredhud, true);
@@ -1458,12 +1487,14 @@ static void M_DrawCrispness2(void)
     M_DrawCrispnessSeparator(crispness_sep_audible, "Audible");
     M_DrawCrispnessItem(crispness_soundfull, "Play sounds in full length", crispy->soundfull, true);
     M_DrawCrispnessItem(crispness_soundfix, "Misc. Sound Fixes", crispy->soundfix, true);
-    M_DrawCrispnessMultiItem(crispness_sndchannels, "Sound Channels", multiitem_sndchannels, crispy->sndchannels, true);
+    M_DrawCrispnessMultiItem(crispness_sndchannels, "Sound Channels", multiitem_sndchannels, snd_channels >> 4, true);
     M_DrawCrispnessItem(crispness_soundmono, "Mono SFX", crispy->soundmono, true);
 
     M_DrawCrispnessSeparator(crispness_sep_navigational, "Navigational");
     M_DrawCrispnessItem(crispness_extautomap, "Extended Automap colors", crispy->extautomap, true);
-    M_DrawCrispnessItem(crispness_automapstats, "Show Level Stats in Automap", crispy->automapstats, true);
+    M_DrawCrispnessMultiItem(crispness_automapstats, "Show Level Stats", multiitem_widgets, crispy->automapstats, true);
+    M_DrawCrispnessMultiItem(crispness_leveltime, "Show Level Time", multiitem_widgets, crispy->leveltime, true);
+    M_DrawCrispnessMultiItem(crispness_playercoords, "Show Player Coords", multiitem_widgets, crispy->playercoords, true);
     M_DrawCrispnessItem(crispness_secretmessage, "Show Revealed Secrets", crispy->secretmessage, true);
 
     M_DrawCrispnessGoto(crispness2_next, "Next Page >");
@@ -1706,9 +1737,9 @@ void M_QuitResponse(int key)
 }
 
 
-static char *M_SelectEndMessage(void)
+static const char *M_SelectEndMessage(void)
 {
-    char **endmsg;
+    const char **endmsg;
 
     if (logical_gamemission == doom)
     {
@@ -1757,6 +1788,21 @@ void M_ChangeSensitivity(int choice)
     }
 }
 
+void M_ChangeSensitivity_x2(int choice)
+{
+    switch(choice)
+    {
+      case 0:
+	if (mouseSensitivity_x2)
+	    mouseSensitivity_x2--;
+	break;
+      case 1:
+	if (mouseSensitivity_x2 < 255) // [crispy] extended range
+	    mouseSensitivity_x2++;
+	break;
+    }
+}
+
 static void M_ChangeSensitivity_y(int choice)
 {
     switch(choice)
@@ -1892,7 +1938,7 @@ M_StartMessage
 //
 // Find string width from hu_font chars
 //
-int M_StringWidth(char* string)
+int M_StringWidth(const char *string)
 {
     size_t             i;
     int             w = 0;
@@ -1901,10 +1947,13 @@ int M_StringWidth(char* string)
     for (i = 0;i < strlen(string);i++)
     {
 	// [crispy] correctly center colorized strings
-	if (string[i] == '\x1b')
+	if (string[i] == cr_esc)
 	{
-	    i++;
-	    continue;
+	    if (string[i+1] >= '0' && string[i+1] <= '0' + CRMAX - 1)
+	    {
+		i++;
+		continue;
+	    }
 	}
 
 	c = toupper(string[i]) - HU_FONTSTART;
@@ -1944,10 +1993,10 @@ void
 M_WriteText
 ( int		x,
   int		y,
-  char*		string)
+  const char *string)
 {
     int		w;
-    char*	ch;
+    const char *ch;
     int		c;
     int		cx;
     int		cy;
@@ -1969,11 +2018,14 @@ M_WriteText
 	    continue;
 	}
 	// [crispy] support multi-colored text
-	if (c == '\x1b')
+	if (c == cr_esc)
 	{
-	    c = *ch++;
-	    dp_translation = cr[(int) (c - '0')];
-	    continue;
+	    if (*ch >= '0' && *ch <= '0' + CRMAX - 1)
+	    {
+		c = *ch++;
+		dp_translation = cr[(int) (c - '0')];
+		continue;
+	    }
 	}
 		
 	c = toupper(c) - HU_FONTSTART;
@@ -2185,71 +2237,73 @@ boolean M_Responder (event_t* ev)
     {
         // Simulate key presses from joystick events to interact with the menu.
 
-	if (ev->data3 < 0)
-	{
-	    key = key_menu_up;
-	    joywait = I_GetTime() + 5;
-	}
-	else if (ev->data3 > 0)
-	{
-	    key = key_menu_down;
-	    joywait = I_GetTime() + 5;
-	}
-		
-	if (ev->data2 < 0)
-	{
-	    key = key_menu_left;
-	    joywait = I_GetTime() + 2;
-	}
-	else if (ev->data2 > 0)
-	{
-	    key = key_menu_right;
-	    joywait = I_GetTime() + 2;
-	}
+        if (menuactive)
+        {
+            if (ev->data3 < 0)
+            {
+                key = key_menu_up;
+                joywait = I_GetTime() + 5;
+            }
+            else if (ev->data3 > 0)
+            {
+                key = key_menu_down;
+                joywait = I_GetTime() + 5;
+            }
+            if (ev->data2 < 0)
+            {
+                key = key_menu_left;
+                joywait = I_GetTime() + 2;
+            }
+            else if (ev->data2 > 0)
+            {
+                key = key_menu_right;
+                joywait = I_GetTime() + 2;
+            }
 
 #define JOY_BUTTON_MAPPED(x) ((x) >= 0)
 #define JOY_BUTTON_PRESSED(x) (JOY_BUTTON_MAPPED(x) && (ev->data1 & (1 << (x))) != 0)
 
-        if (JOY_BUTTON_PRESSED(joybfire))
-        {
-            // Simulate a 'Y' keypress when Doom show a Y/N dialog with Fire button.
-            if (messageToPrint && messageNeedsInput)
+            if (JOY_BUTTON_PRESSED(joybfire))
             {
-                key = key_menu_confirm;
-            }
-            // Simulate pressing "Enter" when we are supplying a save slot name
-            else if (saveStringEnter)
-            {
-                key = KEY_ENTER;
-            }
-            else
-            {
-                // if selecting a save slot via joypad, set a flag
-                if (currentMenu == &SaveDef)
+                // Simulate a 'Y' keypress when Doom show a Y/N dialog with Fire button.
+                if (messageToPrint && messageNeedsInput)
                 {
-                    joypadSave = true;
+                    key = key_menu_confirm;
                 }
-                key = key_menu_forward;
+                // Simulate pressing "Enter" when we are supplying a save slot name
+                else if (saveStringEnter)
+                {
+                    key = KEY_ENTER;
+                }
+                else
+                {
+                    // if selecting a save slot via joypad, set a flag
+                    if (currentMenu == &SaveDef)
+                    {
+                        joypadSave = true;
+                    }
+                    key = key_menu_forward;
+                }
+                joywait = I_GetTime() + 5;
             }
-            joywait = I_GetTime() + 5;
-        }
-        if (JOY_BUTTON_PRESSED(joybuse))
-        {
-            // Simulate a 'N' keypress when Doom show a Y/N dialog with Use button.
-            if (messageToPrint && messageNeedsInput)
+            if (JOY_BUTTON_PRESSED(joybuse))
             {
-                key = key_menu_abort;
+                // Simulate a 'N' keypress when Doom show a Y/N dialog with Use button.
+                if (messageToPrint && messageNeedsInput)
+                {
+                    key = key_menu_abort;
+                }
+                // If user was entering a save name, back out
+                else if (saveStringEnter)
+                {
+                    key = KEY_ESCAPE;
+                }
+                else
+                {
+                    key = key_menu_back;
+                }
+                joywait = I_GetTime() + 5;
             }
-            // If user was entering a save name, back out
-            else if (saveStringEnter)
-            {
-                key = KEY_ESCAPE;
-            }
-            else
-            {
-                key = key_menu_back;
-            }
-            joywait = I_GetTime() + 5;
         }
         if (JOY_BUTTON_PRESSED(joybmenu))
         {
@@ -2477,17 +2531,9 @@ boolean M_Responder (event_t* ev)
         }
         else if (key == key_menu_load)     // Load
         {
-	    // [crispy] forbid New Game and (Quick) Load while recording a demo
-	    if (demorecording)
-	    {
-		S_StartSound(NULL,sfx_oof);
-	    }
-	    else
-	    {
 	    M_StartControlPanel();
 	    S_StartSound(NULL,sfx_swtchn);
 	    M_LoadGame(0);
-	    }
 	    return true;
         }
         else if (key == key_menu_volume)   // Sound Volume
@@ -2524,16 +2570,8 @@ boolean M_Responder (event_t* ev)
         }
         else if (key == key_menu_qload)    // Quickload
         {
-	    // [crispy] forbid New Game and (Quick) Load while recording a demo
-	    if (demorecording)
-	    {
-		S_StartSound(NULL,sfx_oof);
-	    }
-	    else
-	    {
 	    S_StartSound(NULL,sfx_swtchn);
 	    M_QuickLoad();
-	    }
 	    return true;
         }
         else if (key == key_menu_quit)     // Quit DOOM
@@ -2826,9 +2864,9 @@ void M_Drawer (void)
 	y = ORIGHEIGHT/2 - M_StringHeight(messageString) / 2;
 	while (messageString[start] != '\0')
 	{
-	    int foundnewline = 0;
+	    boolean foundnewline = false;
 
-            for (i = 0; i < strlen(messageString + start); i++)
+            for (i = 0; messageString[start + i] != '\0'; i++)
             {
                 if (messageString[start + i] == '\n')
                 {
@@ -2839,7 +2877,7 @@ void M_Drawer (void)
                         string[i] = '\0';
                     }
 
-                    foundnewline = 1;
+                    foundnewline = true;
                     start += i + 1;
                     break;
                 }
@@ -2884,7 +2922,7 @@ void M_Drawer (void)
 	    // [crispy] shade unavailable menu items
 	    if ((currentMenu == &MainDef && i == savegame && (!usergame || gamestate != GS_LEVEL)) ||
 	        (currentMenu == &OptionsDef && i == endgame && (!usergame || netgame)) ||
-	        (currentMenu == &MainDef && i == loadgame && ((netgame && !demoplayback) || demorecording)) ||
+	        (currentMenu == &MainDef && i == loadgame && (netgame && !demoplayback)) ||
 	        (currentMenu == &MainDef && i == newgame && (demorecording || (netgame && !demoplayback))))
 	        dp_translation = cr[CR_DARK];
 
@@ -3047,13 +3085,15 @@ void M_Init (void)
 		LoadDef_y = vstep + captionheight - SHORT(patchl->height) + SHORT(patchl->topoffset);
 		SaveDef_y = vstep + captionheight - SHORT(patchs->height) + SHORT(patchs->topoffset);
 		LoadDef.y = SaveDef.y = vstep + captionheight + vstep + SHORT(patchm->topoffset) - 7; // [crispy] see M_DrawSaveLoadBorder()
+		MouseDef.y = LoadDef.y;
 	}
     }
 
     // [crispy] remove DOS reference from the game quit confirmation dialogs
     if (!M_ParmExists("-nodeh"))
     {
-	char *string, *replace;
+	const char *string;
+	char *replace;
 
 	// [crispy] "i wouldn't leave if i were you.\ndos is much worse."
 	string = doom1_endmsg[3];
@@ -3093,9 +3133,10 @@ static void M_ForceLoadGameResponse(int key)
 	free(savegwarning);
 	free(savewadfilename);
 
-	if (key != key_menu_confirm)
+	if (key != key_menu_confirm || !savemaplumpinfo)
 	{
-		M_EndGameResponse(key_menu_confirm);
+		// [crispy] no need to end game anymore when denied to load savegame
+		//M_EndGameResponse(key_menu_confirm);
 		savewadfilename = NULL;
 
 		// [crispy] reload Load Game menu
@@ -3104,21 +3145,27 @@ static void M_ForceLoadGameResponse(int key)
 		return;
 	}
 
-	savewadfilename = maplumpinfo->wad_file->basename;
+	savewadfilename = (char *)W_WadNameForLump(savemaplumpinfo);
 	gameaction = ga_loadgame;
 }
 
 void M_ForceLoadGame()
 {
 	savegwarning =
+	savemaplumpinfo ?
 	M_StringJoin("This savegame requires the file\n",
 	             crstr[CR_GOLD], savewadfilename, crstr[CR_NONE], "\n",
-	             "to restore ", crstr[CR_GOLD], maplumpinfo->name, crstr[CR_NONE], " .\n\n",
+	             "to restore ", crstr[CR_GOLD], savemaplumpinfo->name, crstr[CR_NONE], " .\n\n",
 	             "Continue to restore from\n",
-	             crstr[CR_GOLD], maplumpinfo->wad_file->basename, crstr[CR_NONE], " ?\n\n",
-	             PRESSYN, NULL);
+	             crstr[CR_GOLD], W_WadNameForLump(savemaplumpinfo), crstr[CR_NONE], " ?\n\n",
+	             PRESSYN, NULL) :
+	M_StringJoin("This savegame requires the file\n",
+	             crstr[CR_GOLD], savewadfilename, crstr[CR_NONE], "\n",
+	             "to restore a map that is\n",
+	             "currently not available!\n\n",
+	             PRESSKEY, NULL) ;
 
-	M_StartMessage(savegwarning, M_ForceLoadGameResponse, true);
+	M_StartMessage(savegwarning, M_ForceLoadGameResponse, savemaplumpinfo != NULL);
 	messageToPrint = 2;
 	S_StartSound(NULL,sfx_swtchn);
 }
