@@ -185,16 +185,11 @@ boolean P_CheckMeleeRange (mobj_t*	actor)
     if (! P_CheckSight (actor, actor->target) )
 	return false;
 							
-    // [crispy] height check for melee attacks (from Hexen)
+    // [crispy] height check for melee attacks
     if (critical->overunder && pl->player)
     {
-	// [crispy] Target is higher than the attacker
-	if (pl->z > actor->z + actor->height)
-	{
-	    return false;
-	}
-	// [crispy] Attacker is higher
-	else if (actor->z > pl->z + pl->height)
+	if (pl->z >= actor->z + actor->height ||
+	    actor->z >= pl->z + pl->height)
 	{
 	    return false;
 	}
@@ -666,10 +661,17 @@ void A_Look (mobj_t* actor)
 	    || actor->type == MT_CYBORG)
 	{
 	    // full volume
-	    S_StartSound (NULL, sound);
+	    // [crispy] prevent from adding up volume
+	    crispy->soundfull ? S_StartSoundOnce (NULL, sound) : S_StartSound (NULL, sound);
 	}
 	else
 	    S_StartSound (actor, sound);
+
+	// [crispy] make seesounds uninterruptible
+	if (crispy->soundfull)
+	{
+		S_UnlinkSound(actor);
+	}
     }
 
     P_SetMobjState (actor, actor->info->seestate);
@@ -1236,7 +1238,11 @@ void A_VileChase (mobj_t* actor)
 
 		    // [crispy] resurrected pools of gore ("ghost monsters") are translucent
 		    if (corpsehit->height == 0 && corpsehit->radius == 0)
+		    {
 		        corpsehit->flags |= MF_TRANSLUCENT;
+		        fprintf(stderr, "A_VileChase: Resurrected ghost monster (%d) at (%d/%d)!\n",
+		                corpsehit->type, corpsehit->x>>FRACBITS, corpsehit->y>>FRACBITS);
+		    }
 
 		    return;
 		}
@@ -1609,7 +1615,8 @@ void A_Scream (mobj_t* actor)
 	|| actor->type == MT_CYBORG)
     {
 	// full volume
-	S_StartSound (NULL, sound);
+	// [crispy] prevent from adding up volume
+	crispy->soundfull ? S_StartSoundOnce (NULL, sound) : S_StartSound (NULL, sound);
     }
     else
 	S_StartSound (actor, sound);
@@ -1928,7 +1935,8 @@ void A_BrainAwake (mobj_t* mo)
 
 void A_BrainPain (mobj_t*	mo)
 {
-    S_StartSound (NULL,sfx_bospn);
+    // [crispy] prevent from adding up volume
+    crispy->soundfull ? S_StartSoundOnce (NULL,sfx_bospn) : S_StartSound (NULL,sfx_bospn);
 }
 
 
@@ -2008,6 +2016,10 @@ void A_BrainSpit (mobj_t*	mo)
 
     // shoot a cube at current target
     targ = braintargets[braintargeton];
+    if (numbraintargets == 0 && false) // [crispy] fixed division by zero
+    {
+        I_Error("A_BrainSpit: numbraintargets was 0 (vanilla crashes here)");
+    }
     braintargeton = (braintargeton+1)%numbraintargets;
 
     // spawn brain missile
