@@ -110,6 +110,7 @@ byte*		rejectmatrix;
 mapthing_t	deathmatchstarts[MAX_DEATHMATCH_STARTS];
 mapthing_t*	deathmatch_p;
 mapthing_t	playerstarts[MAXPLAYERS];
+boolean     playerstartsingame[MAXPLAYERS];
 
 // [crispy] recalculate seg offsets
 // adapted from prboom-plus/src/p_setup.c:474-482
@@ -401,7 +402,8 @@ void P_LoadSectors (int lump)
         ss->interpfloorheight = ss->floorheight;
         ss->oldceilingheight = ss->ceilingheight;
         ss->interpceilingheight = ss->ceilingheight;
-        ss->oldgametic = 0;
+        // [crispy] inhibit sector interpolation during the 0th gametic
+        ss->oldgametic = -1;
     }
 	
     W_ReleaseLumpNum(lump);
@@ -501,12 +503,6 @@ void P_LoadThings (int lump)
     data = W_CacheLumpNum (lump,PU_STATIC);
     numthings = W_LumpLength (lump) / sizeof(mapthing_t);
 	
-    // [crispy] warn about missing thing
-    if (!data || !numthings)
-    {
-	I_Error("P_LoadThings: No things in map!");
-    }
-
     mt = (mapthing_t *)data;
     for (i=0 ; i<numthings ; i++, mt++)
     {
@@ -550,6 +546,18 @@ void P_LoadThings (int lump)
 	P_SpawnMapThing(&spawnthing);
     }
 
+    if (!deathmatch)
+    {
+        for (i = 0; i < MAXPLAYERS; i++)
+        {
+            if (playeringame[i] && !playerstartsingame[i])
+            {
+                I_Error("P_LoadThings: Player %d start missing (vanilla crashes here)", i + 1);
+            }
+            playerstartsingame[i] = false;
+        }
+    }
+
     W_ReleaseLumpNum(lump);
 }
 
@@ -1090,7 +1098,44 @@ const char *skilltable[] =
     "NM"
 };
 
-// [crispy] pointer to the current map lump info struct
+// [crispy] factor out map lump name and number finding into a separate function
+int P_GetNumForMap (int episode, int map, boolean critical)
+{
+    char	lumpname[9];
+    int		lumpnum;
+
+    // find map name
+    if ( gamemode == commercial)
+    {
+	if (map<10)
+	    DEH_snprintf(lumpname, 9, "map0%i", map);
+	else
+	    DEH_snprintf(lumpname, 9, "map%i", map);
+    }
+    else
+    {
+	lumpname[0] = 'E';
+	lumpname[1] = '0' + episode;
+	lumpname[2] = 'M';
+	lumpname[3] = '0' + map;
+	lumpname[4] = 0;
+    }
+
+    // [crispy] special-casing for E1M10 "Sewers" support
+    if (crispy->havee1m10 && episode == 1 && map == 10)
+	DEH_snprintf(lumpname, 9, "E1M10");
+
+    lumpnum = critical ? W_GetNumForName (lumpname) : W_CheckNumForName (lumpname);
+
+    if (nervewadfile && episode != 2 && map <= 9)
+    {
+        lumpnum = W_CheckNumForNameFromTo (lumpname, lumpnum - 1, 0);
+    }
+
+    return lumpnum;
+}
+
+// pointer to the current map lump info struct
 lumpinfo_t *maplumpinfo;
 
 //
@@ -1135,7 +1180,7 @@ P_SetupLevel
     {
         if (gamemission == pack_nerve)
         {
-            gameepisode = 2;
+            episode = gameepisode = 2;
         }
     }
 
@@ -1143,8 +1188,21 @@ P_SetupLevel
     // will be set by player think.
     players[consoleplayer].viewz = 1; 
 
+    // [crispy] stop demo warp mode now
+    if (crispy->demowarp == map)
+    {
+	crispy->demowarp = 0;
+	nodrawers = false;
+	singletics = false;
+    }
+
+    // [crispy] don't load map's default music if loaded from a savegame with MUSINFO data
+    if (!musinfo.from_savegame)
+    {
     // Make sure all sounds are stopped before Z_FreeTags.
     S_Start ();			
+    }
+    musinfo.from_savegame = false;
 
     Z_FreeTags (PU_LEVEL, PU_PURGELEVEL-1);
 
@@ -1154,6 +1212,8 @@ P_SetupLevel
     // if working with a devlopment map, reload it
     W_Reload ();
 
+// [crispy] factor out map lump name and number finding into a separate function
+/*
     // find map name
     if ( gamemode == commercial)
     {
@@ -1171,21 +1231,15 @@ P_SetupLevel
 	lumpname[4] = 0;
     }
 
-    // [crispy] special-casing for E1M10 "Sewers" support
-    if (crispy->havee1m10 && episode == 1 && map == 10)
-	DEH_snprintf(lumpname, 9, "E1M10");
-
     lumpnum = W_GetNumForName (lumpname);
+*/
+    lumpnum = P_GetNumForMap (episode, map, true);
 	
-    if (nervewadfile && gamemission != pack_nerve && map <= 9)
-    {
-        lumpnum = W_CheckNumForNameFromTo (lumpname, lumpnum - 1, 0);
-    }
-
-    // [crispy] pointer to the current map lump info struct
     maplumpinfo = lumpinfo[lumpnum];
+    strncpy(lumpname, maplumpinfo->name, 8);
 
     leveltime = 0;
+    oldleveltime = 0;
 	
     // [crispy] better logging
     {
@@ -1201,7 +1255,7 @@ P_SetupLevel
 	    NULL);
 
 	fprintf(stderr, "P_SetupLevel: %s (%s), %s%s, Time %d:%02d:%02d, Total %d:%02d:%02d, ",
-	    maplumpinfo->name, maplumpinfo->wad_file->basename,
+	    maplumpinfo->name, W_WadNameForLump(maplumpinfo),
 	    skilltable[BETWEEN(0,5,(int) skill+1)], rfn_str,
 	    ltime/3600, (ltime%3600)/60, ltime%60,
 	    ttime/3600, (ttime%3600)/60, ttime%60);
@@ -1250,6 +1304,8 @@ P_SetupLevel
     P_RemoveSlimeTrails();
     // [crispy] fix long wall wobble
     P_SegLengths(false);
+    // [crispy] blinking key or skull in the status bar
+    memset(st_keyorskull, 0, sizeof(st_keyorskull));
 
     bodyqueslot = 0;
     deathmatch_p = deathmatchstarts;
@@ -1290,15 +1346,38 @@ P_SetupLevel
 
     //printf ("free memory: 0x%x\n", Z_FreeMemory());
 
-    // [crispy] stop demo warp mode now
-    if (crispy->demowarp == map)
-    {
-	crispy->demowarp = 0;
-	nodrawers = false;
-	singletics = false;
-    }
 }
 
+// [crispy] height of the spawnstate's first sprite in pixels
+static void P_InitActualHeights (void)
+{
+	int i;
+
+	for (i = 0; i < NUMMOBJTYPES; i++)
+	{
+		state_t *state;
+		spritedef_t *sprdef;
+		spriteframe_t *sprframe;
+		int lump;
+		patch_t *patch;
+
+		state = &states[mobjinfo[i].spawnstate];
+		sprdef = &sprites[state->sprite];
+
+		if (!sprdef->numframes || !(mobjinfo[i].flags & (MF_SOLID|MF_SHOOTABLE)))
+		{
+			mobjinfo[i].actualheight = mobjinfo[i].height;
+			continue;
+		}
+
+		sprframe = &sprdef->spriteframes[state->frame & FF_FRAMEMASK];
+		lump = sprframe->lump[0];
+		patch = W_CacheLumpNum (lump + firstspritelump, PU_CACHE);
+
+		// [crispy] round to the next integer multiple of 8
+		mobjinfo[i].actualheight = ((patch->height + 7) & (~7)) << FRACBITS;
+	}
+}
 
 
 //
@@ -1309,6 +1388,7 @@ void P_Init (void)
     P_InitSwitchList ();
     P_InitPicAnims ();
     R_InitSprites (sprnames);
+    P_InitActualHeights();
 }
 
 
