@@ -1144,6 +1144,8 @@ void P_PlayerInSpecialSector (player_t* player)
 	        S_StartSound(NULL, sfx_id);
 	}
 	player->secretcount++;
+	// [crispy] remember revealed secrets
+	sector->oldspecial = sector->special;
 	sector->special = 0;
 	break;
 			
