@@ -107,14 +107,6 @@ static musicinfo_t *mus_playing = NULL;
 
 int snd_channels = 8;
 
-// [crispy] variable number of sound channels
-static inline int update_snd_channels (void)
-{
-	return (crispy->sndchannels == 2) ? 32 :
-	       (crispy->sndchannels == 1) ? 16 :
-	       8;
-}
-
 //
 // Initializes sound stuff, including volume
 // Sets channels, SFX and music volume,
@@ -150,7 +142,6 @@ void S_Init(int sfxVolume, int musicVolume)
     // (the maximum numer of sounds rendered
     // simultaneously) within zone memory.
     // [crispy] variable number of sound channels
-    snd_channels = update_snd_channels();
     channels = I_Realloc(NULL, snd_channels*sizeof(channel_t));
     sobjs = I_Realloc(NULL, snd_channels*sizeof(degenmobj_t));
 
@@ -324,7 +315,7 @@ void S_Start(void)
 	prevmap = curmap;
     }
 
-    // [crispy] musinfo.items[0] is reserved for the map's default music
+    // [crispy] reset musinfo data at the start of a new map
     memset(&musinfo, 0, sizeof(musinfo));
 
     S_ChangeMusic(mnum, true);
@@ -355,16 +346,19 @@ void S_UnlinkSound(mobj_t *origin)
 {
     int cnum;
 
-    for (cnum=0 ; cnum<snd_channels ; cnum++)
+    if (origin)
     {
-        if (channels[cnum].sfxinfo && channels[cnum].origin == origin)
+        for (cnum=0 ; cnum<snd_channels ; cnum++)
         {
-            degenmobj_t *const sobj = &sobjs[cnum];
-            sobj->x = origin->x;
-            sobj->y = origin->y;
-            sobj->z = origin->z;
-            channels[cnum].origin = (mobj_t *) sobj;
-            break;
+            if (channels[cnum].sfxinfo && channels[cnum].origin == origin)
+            {
+                degenmobj_t *const sobj = &sobjs[cnum];
+                sobj->x = origin->x;
+                sobj->y = origin->y;
+                sobj->z = origin->z;
+                channels[cnum].origin = (mobj_t *) sobj;
+                break;
+            }
         }
     }
 }
@@ -443,6 +437,9 @@ static int S_AdjustSoundParams(mobj_t *listener, mobj_t *source,
     fixed_t        ady;
     angle_t        angle;
 
+    // [crispy] proper sound clipping in Doom 2 MAP08 and The Ultimate Doom E4M8
+    const boolean doom1map8 = (gamemap == 8 && ((gamemode != commercial && gameepisode != 4) || !crispy->soundfix));
+
     // calculate the distance to sound origin
     //  and clip it if necessary
     adx = abs(listener->x - source->x);
@@ -451,8 +448,7 @@ static int S_AdjustSoundParams(mobj_t *listener, mobj_t *source,
     // From _GG1_ p.428. Appox. eucledian distance fast.
     approx_dist = adx + ady - ((adx < ady ? adx : ady)>>1);
 
-    // [crispy] proper sound clipping in non-Doom1 MAP08
-    if ((gamemap != 8 || (crispy->soundfix && gamemode == commercial)) && approx_dist > S_CLIPPING_DIST)
+    if (!doom1map8 && approx_dist > S_CLIPPING_DIST)
     {
         return 0;
     }
@@ -482,8 +478,7 @@ static int S_AdjustSoundParams(mobj_t *listener, mobj_t *source,
     {
         *vol = snd_SfxVolume;
     }
-    // [crispy] proper sound clipping in non-Doom1 MAP08
-    else if (gamemap == 8 && (gamemode != commercial || !crispy->soundfix))
+    else if (doom1map8)
     {
         if (approx_dist > S_CLIPPING_DIST)
         {
@@ -531,7 +526,7 @@ void S_StartSound(void *origin_p, int sfx_id)
     int volume;
 
     // [crispy] play no sounds
-    if (crispy->demowarp)
+    if (nodrawers && singletics)
 	return;
 
     origin = (mobj_t *) origin_p;
@@ -606,7 +601,10 @@ void S_StartSound(void *origin_p, int sfx_id)
     pitch = Clamp(pitch);
 
     // kill old sound
+    if (!crispy->soundfull || origin || gamestate != GS_LEVEL)
+    {
     S_StopSound(origin);
+    }
 
     // try to find a channel
     cnum = S_GetChannel(origin, sfx);
@@ -795,7 +793,7 @@ void S_ChangeMusic(int musicnum, int looping)
     musinfo.current_item = -1;
 
     // [crispy] play no music if this is not the right map
-    if (crispy->demowarp && (gamestate != GS_LEVEL || crispy->demowarp != gamemap))
+    if (nodrawers && singletics)
 	return;
 
     // [crispy] restart current music if IDMUS00 is entered
@@ -813,7 +811,7 @@ void S_ChangeMusic(int musicnum, int looping)
         const int intro = W_GetNumForName("D_INTRO"),
                   introa = W_GetNumForName("D_INTROA");
         // [crispy] if D_INTRO is from a PWAD, and D_INTROA is from a different WAD file, play the former
-        if (lumpinfo[intro]->wad_file->iwad || (lumpinfo[intro]->wad_file == lumpinfo[introa]->wad_file))
+        if (W_IsIWADLump(lumpinfo[intro]) || (lumpinfo[intro]->wad_file == lumpinfo[introa]->wad_file))
         {
         musicnum = mus_introa;
         }
@@ -888,7 +886,7 @@ void S_ChangeMusInfoMusic (int lumpnum, int looping)
     prevmap = -1;
 
     // [crispy] play no music if this is not the right map
-    if (crispy->demowarp && (gamestate != GS_LEVEL || crispy->demowarp != gamemap))
+    if (nodrawers && singletics)
     {
 	musinfo.current_item = lumpnum;
 	return;
@@ -955,7 +953,12 @@ void S_UpdateSndChannels (void)
 		}
 	}
 
-	snd_channels = update_snd_channels();
+	snd_channels <<= 1;
+	if (snd_channels > 32)
+	{
+		snd_channels = 8;
+	}
+
 	channels = I_Realloc(channels, snd_channels * sizeof(channel_t));
 	sobjs = I_Realloc(sobjs, snd_channels * sizeof(degenmobj_t));
 
