@@ -353,6 +353,8 @@ static st_binicon_t	w_armsbg;
 
 // weapon ownership widgets
 static st_multicon_t	w_arms[6];
+// [crispy] show SSG availability in the Shotgun slot of the arms widget
+static int st_shotguns;
 
 // face status widget
 static st_multicon_t	w_faces; 
@@ -388,6 +390,8 @@ static int	st_faceindex = 0;
 
 // holds key-type for each key box on bar
 static int	keyboxes[3]; 
+// [crispy] blinking key or skull in the status bar
+int		st_keyorskull[3];
 
 // a random number per tick
 static int	st_randomnumber;  
@@ -604,6 +608,21 @@ static int ST_cheat_spechits()
     return (speciallines);
 }
 
+// [crispy] only give available weapons
+static boolean WeaponAvailable (int w)
+{
+	if (w < 0 || w >= NUMWEAPONS)
+	    return false;
+
+	if (w == wp_supershotgun && !crispy->havessg)
+	    return false;
+
+	if ((w == wp_bfg || w == wp_plasma) && gamemode == shareware)
+	    return false;
+
+	return true;
+}
+
 // Respond to keyboard input events,
 //  intercept cheats.
 boolean
@@ -687,6 +706,7 @@ ST_Responder (event_t* ev)
 	}
 
 	for (i=0;i<NUMWEAPONS;i++)
+	 if (WeaponAvailable(i)) // [crispy] only give available weapons
 	  plyr->weaponowned[i] = true;
 	
 	for (i=0;i<NUMAMMO;i++)
@@ -712,6 +732,7 @@ ST_Responder (event_t* ev)
 	}
 
 	for (i=0;i<NUMWEAPONS;i++)
+	 if (WeaponAvailable(i)) // [crispy] only give available weapons
 	  plyr->weaponowned[i] = true;
 	
 	for (i=0;i<NUMAMMO;i++)
@@ -746,7 +767,9 @@ ST_Responder (event_t* ev)
 	  S_ChangeMusic(0, 2);
 	}
 	else
-	if (gamemode == commercial || gameversion < exe_ultimate)
+	// [JN] Fixed: using a proper IDMUS selection for shareware
+	// and registered game versions.
+	if (gamemode == commercial /* || gameversion < exe_ultimate */ )
 	{
 	  musnum = mus_runnin + (buf[0]-'0')*10 + buf[1]-'0' - 1;
 	  
@@ -803,7 +826,7 @@ ST_Responder (event_t* ev)
 	{
 	  if (!plyr->powers[i])
 	    P_GivePower( plyr, i);
-	  else if (i!=pw_strength)
+	  else if (i!=pw_strength && i!=pw_allmap) // [crispy] disable full Automap
 	    plyr->powers[i] = 1;
 	  else
 	    plyr->powers[i] = 0;
@@ -833,8 +856,6 @@ ST_Responder (event_t* ev)
       // 'mypos' for player position
       else if (cht_CheckCheat(&cheat_mypos, ev->data2))
       {
-        // [crispy] show (human readable) map coordinates
-        // in the upper right corner (like automap stats)
 /*
         static char buf[ST_MSGWIDTH];
         M_snprintf(buf, sizeof(buf), "ang=0x%x;x,y=(0x%x,0x%x)",
@@ -843,7 +864,8 @@ ST_Responder (event_t* ev)
                    players[consoleplayer].mo->y);
         plyr->message = buf;
 */
-        plyr->powers[pw_mapcoords] ^= 1;
+        // [crispy] extra high precision IDMYPOS variant, updates for 10 seconds
+        plyr->powers[pw_mapcoords] = 10*TICRATE;
       }
 
 // [crispy] now follow "critical" Crispy Doom specific cheats
@@ -944,13 +966,8 @@ ST_Responder (event_t* ev)
 	cht_GetParam(&cheat_weapon, buf);
 	w = *buf - '1';
 
-	if (w < 0 || w >= NUMWEAPONS)
-	    return false;
-
-	if (w == wp_supershotgun && !crispy->havessg)
-	    return false;
-
-	if ((w == wp_bfg || w == wp_plasma) && gamemode == shareware)
+	// [crispy] only give available weapons
+	if (!WeaponAvailable(w))
 	    return false;
 
 	// make '1' apply beserker strength toggle
@@ -1073,6 +1090,9 @@ ST_Responder (event_t* ev)
         }
       }
 
+  // [crispy] only fix episode/map if it doesn't exist
+  if (P_GetNumForMap(epsd, map, false) < 0)
+  {
       // Catch invalid maps.
       if (gamemode != commercial)
       {
@@ -1138,10 +1158,15 @@ ST_Responder (event_t* ev)
               return false;
           }
       }
+  }
 
+      // [crispy] prevent idclev to nonexistent levels exiting the game
+      if (P_GetNumForMap(epsd, map, false) >= 0)
+      {
       // So be it.
       plyr->message = DEH_String(STSTR_CLEV);
       G_DeferedInitNew(gameskill, epsd, map);
+      }
 
       // [crispy] eat key press, i.e. don't change weapon upon level change
       return true;
@@ -1244,6 +1269,8 @@ void ST_updateFaceWidget(void)
 	    // [crispy] show "Ouch Face" as intended
 	    if (st_oldhealth - plyr->health > ST_MUCHPAIN)
 	    {
+		// [crispy] raise "Ouch Face" priority
+		priority = 8;
 		st_facecount = ST_TURNCOUNT;
 		faceindex = ST_OUCHOFFSET;
 	    }
@@ -1397,6 +1424,35 @@ void ST_updateWidgets(void)
 
 	if (plyr->cards[i+3])
 	    keyboxes[i] = (keyboxes[i] == -1) ? i+3 : i+6; // [crispy] support combined card and skull keys
+
+	// [crispy] blinking key or skull in the status bar
+	if (plyr->tryopen[i])
+	{
+#if defined(CRISPY_KEYBLINK_WITH_SOUND)
+		if (!(plyr->tryopen[i] & (2*KEYBLINKMASK-1)))
+		{
+			S_StartSound(NULL, sfx_itemup);
+		}
+#endif
+#if defined(CRISPY_KEYBLINK_IN_CLASSIC_HUD)
+		if (screenblocks < CRISPY_HUD && !(plyr->tryopen[i] & (KEYBLINKMASK-1)))
+		{
+			st_firsttime = true;
+		}
+#endif
+		plyr->tryopen[i]--;
+#if !defined(CRISPY_KEYBLINK_IN_CLASSIC_HUD)
+		if (screenblocks >= CRISPY_HUD)
+#endif
+		{
+			keyboxes[i] = (plyr->tryopen[i] & KEYBLINKMASK) ? i + st_keyorskull[i] : -1;
+		}
+
+		if (!plyr->tryopen[i])
+		{
+			w_keyboxes[i].oldinum = -1;
+		}
+	}
     }
 
     // refresh everything if this is him coming back to life
@@ -1590,12 +1646,12 @@ static byte* ST_WidgetColor(int i)
                 plyr->powers[pw_invulnerability])
                 return cr[CR_GRAY];
 	    // [crispy] color by armor type
-	    else if (plyr->armortype == 0)
-                return cr[CR_RED];
+	    else if (plyr->armortype >= 2)
+                return cr[CR_BLUE];
 	    else if (plyr->armortype == 1)
                 return cr[CR_GREEN];
-	    else if (plyr->armortype == 2)
-                return cr[CR_BLUE];
+	    else if (plyr->armortype == 0)
+                return cr[CR_RED];
 /*
             // [crispy] alternatively, color by armor points
             int armor = plyr->armorpoints;
@@ -1722,6 +1778,9 @@ void ST_drawWidgets(boolean refresh)
     STlib_updateBinIcon(&w_armsbg, refresh);
     }
 
+    // [crispy] show SSG availability in the Shotgun slot of the arms widget
+    st_shotguns = plyr->weaponowned[wp_shotgun] | plyr->weaponowned[wp_supershotgun];
+
     for (i=0;i<6;i++)
 	STlib_updateMultIcon(&w_arms[i], refresh || screenblocks >= CRISPY_HUD);
 
@@ -1998,6 +2057,8 @@ void ST_createWidgets(void)
                            &plyr->weaponowned[i+1],
                            &st_armson);
     }
+    // [crispy] show SSG availability in the Shotgun slot of the arms widget
+    w_arms[1].inum = &st_shotguns;
 
     // frags sum
     STlib_initNum(&w_frags,
@@ -2167,7 +2228,7 @@ void ST_Init (void)
     }
 
     ST_loadData();
-    st_backing_screen = (pixel_t *) Z_Malloc((ST_WIDTH << hires) * (ST_HEIGHT << hires) * sizeof(*st_backing_screen), PU_STATIC, 0);
+    st_backing_screen = (pixel_t *) Z_Malloc((ST_WIDTH << 1) * (ST_HEIGHT << 1) * sizeof(*st_backing_screen), PU_STATIC, 0);
 }
 
 // [crispy] Demo Timer widget
@@ -2181,11 +2242,11 @@ void ST_DrawDemoTimer (const int time)
 	n = M_snprintf(buffer, sizeof(buffer), "%02i %02i %02i",
 	               secs / 60, secs % 60, time % TICRATE);
 
-	x = (viewwindowx >> hires) + (scaledviewwidth >> hires);
+	x = (viewwindowx >> crispy->hires) + (scaledviewwidth >> crispy->hires);
 
 	// [crispy] draw the Demo Timer widget with gray numbers
 	dp_translation = cr[CR_GRAY];
-	dp_translucent = true;
+	dp_translucent = (gamestate == GS_LEVEL);
 
 	while (n-- > 0)
 	{
@@ -2195,7 +2256,7 @@ void ST_DrawDemoTimer (const int time)
 
 		if (c >= 0 && c <= 9)
 		{
-			V_DrawPatch(x, viewwindowy >> hires, shortnum[c]);
+			V_DrawPatch(x, viewwindowy >> crispy->hires, shortnum[c]);
 		}
 	}
 
