@@ -43,6 +43,7 @@
 // Needs access to LFB.
 #include "v_video.h"
 
+#include "st_stuff.h" // [crispy] ST_DrawDemoTimer()
 #include "wi_stuff.h"
 
 //
@@ -1496,7 +1497,7 @@ static boolean WI_drawParTime (void)
 	boolean result = true;
 
 	// [crispy] PWADs have no par times (including The Master Levels)
-	if (!maplumpinfo->wad_file->iwad)
+	if (!W_IsIWADLump(maplumpinfo))
 	{
 		result = false;
 	}
@@ -1595,6 +1596,13 @@ void WI_drawStats(void)
 	// [crispy] choose x-position depending on width of time string
 	WI_drawTime((wide ? ORIGWIDTH : ORIGWIDTH/2) - SP_TIMEX, SP_TIMEY + 16, ttime, false);
     }
+
+    // [crispy] demo timer widget
+    if ((demoplayback && (crispy->demotimer & DEMOTIMER_PLAYBACK)) ||
+        (demorecording && (crispy->demotimer & DEMOTIMER_RECORD)))
+    {
+	ST_DrawDemoTimer(leveltime);
+    }
 }
 
 void WI_checkForAccelerate(void)
