@@ -49,7 +49,6 @@
 #include "w_wad.h"
 #include "z_zone.h"
 
-int hires = 1;
 int SCREENWIDTH, SCREENHEIGHT, SCREENHEIGHT_4_3;
 
 // These are (1) the window (or the full screen) that our game is rendered to
@@ -115,8 +114,8 @@ int video_display = 0;
 
 // Screen width and height, from configuration file.
 
-int window_width = MAXWIDTH; // hires
-int window_height = MAXHEIGHT_4_3; // hires
+int window_width = 800;
+int window_height = 600;
 
 // Fullscreen mode, 0x0 for SDL_WINDOW_FULLSCREEN_DESKTOP.
 
@@ -613,6 +612,8 @@ static void CreateUpscaledTexture(boolean force)
     int h_upscale, w_upscale;
     static int h_upscale_old, w_upscale_old;
 
+    SDL_Texture *new_texture, *old_texture;
+
     // Get the size of the renderer output. The units this gives us will be
     // real world pixels, which are not necessarily equivalent to the screen's
     // window size (because of highdpi).
@@ -668,24 +669,31 @@ static void CreateUpscaledTexture(boolean force)
     h_upscale_old = h_upscale;
     w_upscale_old = w_upscale;
 
-    if (texture_upscaled)
-    {
-        SDL_DestroyTexture(texture_upscaled);
-    }
-
     // Set the scaling quality for rendering the upscaled texture to "linear",
     // which looks much softer and smoother than "nearest" but does a better
     // job at downscaling from the upscaled texture to screen.
 
     SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");
 
-    texture_upscaled = SDL_CreateTexture(renderer,
+    new_texture = SDL_CreateTexture(renderer,
                                 pixel_format,
                                 SDL_TEXTUREACCESS_TARGET,
                                 w_upscale*SCREENWIDTH,
                                 h_upscale*SCREENHEIGHT);
+
+    old_texture = texture_upscaled;
+    texture_upscaled = new_texture;
+
+    if (old_texture != NULL)
+    {
+        SDL_DestroyTexture(old_texture);
+    }
 }
 
+// [AM] Fractional part of the current tic, in the half-open
+//      range of [0.0, 1.0).  Used for interpolation.
+fixed_t fractionaltic;
+
 //
 // I_FinishUpdate
 //
@@ -827,6 +835,12 @@ void I_FinishUpdate (void)
 
     SDL_RenderPresent(renderer);
 
+    // [AM] Figure out how far into the current tic we're in as a fixed_t.
+    if (crispy->uncapped)
+    {
+	fractionaltic = I_GetTimeMS() * TICRATE % 1000 * FRACUNIT / 1000;
+    }
+
     // Restore background and undo the disk indicator, if it was drawn.
     V_RestoreDiskBackground();
 }
@@ -845,11 +859,43 @@ void I_ReadScreen (pixel_t* scr)
 // I_SetPalette
 //
 // [crispy] intermediate gamma levels
-byte **gamma2table = NULL;
+static byte **gamma2table;
+static void I_SetGammaTable (void)
+{
+	int i;
+
+	gamma2table = malloc(9 * sizeof(*gamma2table));
+
+	// [crispy] 5 original gamma levels
+	for (i = 0; i < 5; i++)
+	{
+		gamma2table[2*i] = (byte *)gammatable[i];
+	}
+
+	// [crispy] 4 intermediate gamma levels
+	for (i = 0; i < 4; i++)
+	{
+		int j;
+
+		gamma2table[2*i+1] = malloc(256 * sizeof(**gamma2table));
+
+		for (j = 0; j < 256; j++)
+		{
+			gamma2table[2*i+1][j] = (gamma2table[2*i][j] + gamma2table[2*i+2][j]) / 2;
+		}
+	}
+}
+
 void I_SetPalette (byte *doompalette)
 {
     int i;
 
+    // [crispy] intermediate gamma levels
+    if (!gamma2table)
+    {
+        I_SetGammaTable();
+    }
+
     for (i=0; i<256; ++i)
     {
         // Zero out the bottom two bits of each channel - the PC VGA
@@ -1009,8 +1055,6 @@ void I_GraphicsCheckCommandLine(void)
     if (i > 0)
     {
         window_width = atoi(myargv[i + 1]);
-        window_height = window_width * 2;
-        AdjustWindowSize();
         fullscreen = false;
     }
 
@@ -1026,8 +1070,6 @@ void I_GraphicsCheckCommandLine(void)
     if (i > 0)
     {
         window_height = atoi(myargv[i + 1]);
-        window_width = window_height * 2;
-        AdjustWindowSize();
         fullscreen = false;
     }
 
@@ -1178,7 +1220,7 @@ void I_GetWindowPosition(int *x, int *y, int w, int h)
     }
 }
 
-void SetVideoMode(void) // [crispy] un-static
+static void SetVideoMode(void)
 {
     int w, h;
     int x, y;
@@ -1215,6 +1257,15 @@ void SetVideoMode(void) // [crispy] un-static
         }
     }
 
+    // Running without window decorations is potentially useful if you're
+    // playing in three window mode and want to line up three game windows
+    // next to each other on a single desktop.
+    // Deliberately not documented because I'm not sure how useful this is yet.
+    if (M_ParmExists("-borderless"))
+    {
+        window_flags |= SDL_WINDOW_BORDERLESS;
+    }
+
     I_GetWindowPosition(&x, &y, w, h);
 
     // Create window and renderer contexts. We set the window title
@@ -1253,7 +1304,7 @@ void SetVideoMode(void) // [crispy] un-static
     // Turn on vsync if we aren't in a -timedemo
     if (!singletics && mode.refresh_rate > 0)
     {
-      if (crispy->uncapped != UNCAPPED_ON) // [crispy] uncapped vsync
+      if (crispy->vsync) // [crispy] uncapped vsync
       {
         renderer_flags |= SDL_RENDERER_PRESENTVSYNC;
       }
@@ -1263,12 +1314,15 @@ void SetVideoMode(void) // [crispy] un-static
     {
         renderer_flags |= SDL_RENDERER_SOFTWARE;
         renderer_flags &= ~SDL_RENDERER_PRESENTVSYNC;
-        crispy->uncapped = UNCAPPED_OFF;
+        crispy->vsync = false;
     }
 
     if (renderer != NULL)
     {
         SDL_DestroyRenderer(renderer);
+        // all associated textures get destroyed
+        texture = NULL;
+        texture_upscaled = NULL;
     }
 
     renderer = SDL_CreateRenderer(screen, -1, renderer_flags);
@@ -1305,6 +1359,12 @@ void SetVideoMode(void) // [crispy] un-static
 
     // Create the 8-bit paletted and the 32-bit RGBA screenbuffer surfaces.
 
+    if (screenbuffer != NULL)
+    {
+        SDL_FreeSurface(screenbuffer);
+        screenbuffer = NULL;
+    }
+
     if (screenbuffer == NULL)
     {
         screenbuffer = SDL_CreateRGBSurface(0,
@@ -1315,6 +1375,13 @@ void SetVideoMode(void) // [crispy] un-static
 
     // Format of argbbuffer must match the screen pixel format because we
     // import the surface data into the texture.
+
+    if (argbbuffer != NULL)
+    {
+        SDL_FreeSurface(argbbuffer);
+        argbbuffer = NULL;
+    }
+
     if (argbbuffer == NULL)
     {
         SDL_PixelFormatEnumToMasks(pixel_format, &unused_bpp,
@@ -1388,7 +1455,7 @@ void I_InitGraphics(void)
     }
 
     // [crispy] run-time variable high-resolution rendering
-    if ((hires = crispy->hires))
+    if (crispy->hires)
     {
         SCREENWIDTH = MAXWIDTH;
         SCREENHEIGHT = MAXHEIGHT;
@@ -1406,7 +1473,7 @@ void I_InitGraphics(void)
     // [crispy] (re-)initialize resolution-agnostic patch drawing
     V_Init();
 
-    if (aspect_ratio_correct)
+    if (aspect_ratio_correct == 1)
     {
         actualheight = SCREENHEIGHT_4_3;
     }
@@ -1467,6 +1534,128 @@ void I_InitGraphics(void)
     I_AtExit(I_ShutdownGraphics, true);
 }
 
+// [crispy] re-initialize only the parts of the rendering stack that are really necessary
+
+void I_ReInitGraphics (int reinit)
+{
+	// [crispy] re-set rendering resolution and re-create framebuffers
+	if (reinit & REINIT_FRAMEBUFFERS)
+	{
+		unsigned int rmask, gmask, bmask, amask;
+		int unused_bpp;
+
+		if (crispy->hires)
+		{
+			SCREENWIDTH = MAXWIDTH;
+			SCREENHEIGHT = MAXHEIGHT;
+			SCREENHEIGHT_4_3 = MAXHEIGHT_4_3;
+		}
+		else
+		{
+			SCREENWIDTH = ORIGWIDTH;
+			SCREENHEIGHT = ORIGHEIGHT;
+			SCREENHEIGHT_4_3 = ORIGHEIGHT_4_3;
+		}
+		blit_rect.w = SCREENWIDTH;
+		blit_rect.h = SCREENHEIGHT;
+
+		// [crispy] re-initialize resolution-agnostic patch drawing
+		V_Init();
+
+		SDL_FreeSurface(screenbuffer);
+		screenbuffer = SDL_CreateRGBSurface(0,
+				                    SCREENWIDTH, SCREENHEIGHT, 8,
+				                    0, 0, 0, 0);
+
+		// [crispy] re-set the framebuffer pointer
+		I_VideoBuffer = screenbuffer->pixels;
+		V_RestoreBuffer();
+
+		SDL_FreeSurface(argbbuffer);
+		SDL_PixelFormatEnumToMasks(pixel_format, &unused_bpp,
+		                           &rmask, &gmask, &bmask, &amask);
+		argbbuffer = SDL_CreateRGBSurface(0,
+		                                  SCREENWIDTH, SCREENHEIGHT, 32,
+		                                  rmask, gmask, bmask, amask);
+
+		// [crispy] it will get re-created below with the new resolution
+		SDL_DestroyTexture(texture);
+	}
+
+	// [crispy] re-create renderer
+	if (reinit & REINIT_RENDERER)
+	{
+		SDL_RendererInfo info = {0};
+		int flags;
+
+		SDL_GetRendererInfo(renderer, &info);
+		flags = info.flags;
+
+		if (crispy->vsync && !(flags & SDL_RENDERER_SOFTWARE))
+		{
+			flags |= SDL_RENDERER_PRESENTVSYNC;
+		}
+		else
+		{
+			flags &= ~SDL_RENDERER_PRESENTVSYNC;
+		}
+
+		SDL_DestroyRenderer(renderer);
+		renderer = SDL_CreateRenderer(screen, -1, flags);
+		SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
+
+		// [crispy] the texture gets destroyed in SDL_DestroyRenderer(), force its re-creation
+		texture_upscaled = NULL;
+	}
+
+	// [crispy] re-create textures
+	if (reinit & REINIT_TEXTURES)
+	{
+		SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "nearest");
+
+		texture = SDL_CreateTexture(renderer,
+		                            pixel_format,
+		                            SDL_TEXTUREACCESS_STREAMING,
+		                            SCREENWIDTH, SCREENHEIGHT);
+
+		// [crispy] force its re-creation
+		CreateUpscaledTexture(true);
+	}
+
+	// [crispy] re-set logical rendering resolution
+	if (reinit & REINIT_ASPECTRATIO)
+	{
+		if (aspect_ratio_correct == 1)
+		{
+			actualheight = SCREENHEIGHT_4_3;
+		}
+		else
+		{
+			actualheight = SCREENHEIGHT;
+		}
+
+		if (aspect_ratio_correct || integer_scaling)
+		{
+			SDL_RenderSetLogicalSize(renderer,
+			                         SCREENWIDTH,
+			                         actualheight);
+		}
+		else
+		{
+			SDL_RenderSetLogicalSize(renderer, 0, 0);
+		}
+
+		#if SDL_VERSION_ATLEAST(2, 0, 5)
+		SDL_RenderSetIntegerScale(renderer, integer_scaling);
+		#endif
+	}
+
+	// [crispy] adjust the window size and re-set the palette
+	need_resize = true;
+}
+
+// [crispy] take screenshot of the rendered image
+
 void I_RenderReadPixels(byte **data, int *w, int *h, int *p)
 {
 	SDL_Rect rect;
@@ -1518,6 +1707,20 @@ void I_RenderReadPixels(byte **data, int *w, int *h, int *p)
 	format = SDL_AllocFormat(png_format);
 	temp = rect.w * format->BytesPerPixel; // [crispy] pitch
 
+	// [crispy] As far as I understand the issue, SDL_RenderPresent()
+	// may return early, i.e. before it has actually finished rendering the
+	// current texture to screen -- from where we want to capture it.
+	// However, it does never return before it has finished rendering the
+	// *previous* texture.
+	// Thus, we add a second call to SDL_RenderPresent() here to make sure
+	// that it has at least finished rendering the previous texture, which
+	// already contains the scene that we actually want to capture.
+	if (crispy->post_rendering_hook)
+	{
+		SDL_RenderCopy(renderer, crispy->smoothscaling ? texture_upscaled : texture, NULL, NULL);
+		SDL_RenderPresent(renderer);
+	}
+
 	// [crispy] allocate memory for screenshot image
 	pixels = malloc(rect.h * temp);
 	SDL_RenderReadPixels(renderer, &rect, format->format, pixels, temp);
