@@ -130,6 +130,12 @@ static default_t	doom_defaults_list[] =
 
     CONFIG_VARIABLE_INT(mouse_sensitivity),
 
+    //!
+    // Horizontal mouse sensitivity (strafe)
+    //
+
+    CONFIG_VARIABLE_INT(mouse_sensitivity_x2),
+
     //!
     // Vertical mouse sensitivity
     //
@@ -1550,13 +1556,13 @@ static default_t extra_defaults_list[] =
     CONFIG_VARIABLE_KEY(key_map_clearmark),
 
     //!
-    // Key to toogle the overlay mode when in the map view.
+    // Key to toggle the overlay mode when in the map view.
     //
 
     CONFIG_VARIABLE_KEY(key_map_overlay),
 
     //!
-    // Key to toogle the rotate mode when in the map view.
+    // Key to toggle the rotate mode when in the map view.
     //
 
     CONFIG_VARIABLE_KEY(key_map_rotate),
@@ -1790,7 +1796,7 @@ static default_t extra_defaults_list[] =
     //!
     // @game doom
     //
-    // Show additional level statistics in the automap.
+    // Show additional level statistics.
     //
 
     CONFIG_VARIABLE_INT(crispy_automapstats),
@@ -1939,6 +1945,14 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(crispy_jump),
 
+    //!
+    // @game doom
+    //
+    // Show level time.
+    //
+
+    CONFIG_VARIABLE_INT(crispy_leveltime),
+
     //!
     // @game doom
     //
@@ -1971,6 +1985,14 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(crispy_pitch),
 
+    //!
+    // @game doom
+    //
+    // Show player coordinates.
+    //
+
+    CONFIG_VARIABLE_INT(crispy_playercoords),
+
     //!
     // @game doom
     //
